#include <stdio.h>
#include <stdlib.h>
#include <math.h>

  /* Sve particije - Varijacija 2:
       Svi rastavi zadanog prirodnog broja n u sumu
     nepadajucih (prirodnih) pribrojnika, s tim da je
     razlika susjednih pribrojnika barem k, gdje je
     k >= 0 zadani cijeli broj.
     Dozvoljen je i samo jedan pribrojnik.

     Rekurzivno generira (pise, broji i zbraja) sve particije.
     Osnovni rekurzivni algoritam (poput parts_1.c, par0_par.c).

     Ucitava (prirodne) brojeve n i k,
     ispisuje sve opisane particije broja n.
     Na kraju, ispisuje broj opisanih particija p_2(n, k)
     i broj rekurzivnih poziva funkcije.

     Koristi globalno dinamicki alocirano polje pribr za
     pribrojnike u particiji i globalnu varijablu ind_pribr
     za indeks (trenutno) zadnjeg postavljenog pribrojnika u polju.
  */

    /* Globalni pokazivac na dinamicko polje pribrojnika. */

int *pribr;

    /* Globalna varijabla za indeks zadnjeg postavljenog pribrojnika
       (zato da NE mora biti argument rekurzivne funkcije). */

int ind_pribr;


    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;


    /* Funkcija za ispis jedne (zadnje) nadjene particije.
       Koristi globalno polje pribr i globalnu varijablu
       ind_pribr za indeks zadnjeg pribrojnika u polju.
         Inicijalizacija prije vanjskog poziva rekurzivne
       funkcije je ind_pribr = -1, kao signal da jos nema
       postavljenih pribrojnika. */

void ispis_particije(void)
{
    int i;

        /* Provjera indeksa zadnjeg pribrojnika (za svaki slucaj). */
    if (ind_pribr < 0) return;

        /* Ispis particije, sigurno je ind_pribr >= 0. */
    printf("  %2d", pribr[0]);
    for (i = 1; i <= ind_pribr; ++i)
        printf(" + %2d", pribr[i]);
    printf("\n");
}

    /* Nadji sve opisane particije broja suma u kojima je
       prvi (sljedeci) pribrojnik barem jednak broju prvi. */

    /* U trenutku poziva funkcije (ulaza u funkciju), globalni
       ind_pribr = indeks zadnjeg (do sada) postavljenog pribrojnika
       u "globalnoj" particiji polaznog broja. Sljedeci pribrojnik
       se postavlja unutar ovog poziva funkcije. */

int particije_2(int suma, int k, int prvi)
{
    int i, broj = 0;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Provjera kraja rekurzije i jesmo li nasli rastav. */
    if (suma == 0) {

            /* Ispis nadjene particije, sigurno je ind_pribr >= 0. */
        ispis_particije();

            /* Povratak za zavrsenu particiju. */
        return 1;
    }

        /* Povecaj globalni ind_pribr za sljedeci pribrojnik,
           koji se mijenja u sljedecoj petlji po i. */
    ++ind_pribr;

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           uz ogranicenje prvi <= i <= suma:
           - preostala suma je suma - i,
           - sljedeci prvi je barem i + k. */

    for (i = prvi; i <= suma; ++i) {
            /* Postavljanje sljedeceg pribrojnika (i) u polje,
               rekurzivni poziv nakon dodavanja tog pribrojnika */
        pribr[ind_pribr] = i;
        broj += particije_2(suma - i, k, i + k);
    }

        /* Smanji globalni ind_pribr za povratak. */
    --ind_pribr;

    return broj;
}

    /* Nerekurzivna funkcija za racunanje p_2(n, k).
       Zove rekurzivnu funkciju particije_2,
       s dodatnim argumentom prvi = 1. */

int p_2(int n, int k)
{
        /* Inicijalizacija globalnog indeksa zadnjeg (do sada)
           postavljenog pribrojnika na -1 (jos ih nema). */
    ind_pribr = -1;

        /* Poziv rekurzivne funkcije particije_2, s prvi = 1. */
    return particije_2(n, k, 1);
}

int main(void)
{
    int n, k, max_pribr, broj;

    printf(" Upisi prirodni broj n i cijeli broj k >= 0: ");
    scanf("%d%d", &n, &k);

        /* Najveci moguci broj pribrojnika max_pribr mozemo naci tako
           da stavljamo najmanje moguce pribrojnike (uz zadane uvjete)
           sve dok mozemo, tj. iz
             1 + (1+k) + (1+2*k) + ... + (1 + (max_pribr - 1)*k) <= n,
             1 + (1+k) + (1+2*k) + ... + (1 + max_pribr*k) > n.
           Dovoljno je uzeti prvu nejednadzbu i naci najveci takav broj.
           Ako je k > 0, dobivamo kvadratnu jednadzbu za max_pribr
             k*(max_pribr)^2 - (k - 2)*max_pribr - 2*n <= 0,
           odakle izlazi
             max_pribr <= ( k - 2 + sqrt( (k - 2)^2 + 8*k*n ) ) / 2.
           Ako je k = 0, onda odmah izlazi max_br <= n.
        */
    if (k > 0)
        max_pribr = (int) (0.5 * ( k - 2
                            + sqrt( (k - 2.0)*(k - 2.0) + 8.0*k*n ) ));
    else
        max_pribr = n;

        /* Alokacija polja za pribrojnike. */
    if ((pribr = (int*) malloc(max_pribr * sizeof(int))) == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(1);
    }

    broj_poziva = 0u;

    printf("\n\n");
    printf(" Sve trazene particije broja %d su:\n\n", n);

        /* Poziv funkcije - pise i broji sve particije. */
    broj = p_2(n, k);

        /* Zavrsni ispis broja particija i poziva. */
    printf("\n");
    printf(" Broj particija p_2(%d, %d) = %d\n",
           n, k, broj);
    printf(" Broj poziva funkcije = %u\n",
           broj_poziva);

        /* Dealokacija (oslobadjanje) polja za pribrojnike. */
    free(pribr);

    return 0;
}
