#include <stdio.h>
#include <stdlib.h>

  /* Sve particije - Varijacija 3:
       Svi rastavi zadanog prirodnog broja n u sumu
     od tocno k nepadajucih (prirodnih) pribrojnika,
     gdje je k zadani prirodan broj.

     Rekurzivno generira (pise, broji i zbraja) sve particije.
     Osnovni rekurzivni algoritam (poput parts_1.c, par0_par.c).

     Ucitava (prirodne) brojeve n i k,
     ispisuje sve opisane particije broja n.
     Na kraju, ispisuje broj opisanih particija p_3(n, k)
     i broj rekurzivnih poziva funkcije.

     Koristi globalno dinamicki alocirano polje pribr za
     pribrojnike u particiji i globalnu varijablu ind_pribr
     za indeks (trenutno) zadnjeg postavljenog pribrojnika u polju.
  */

    /* Globalni pokazivac na dinamicko polje pribrojnika. */

int *pribr;

    /* Globalna varijabla za indeks zadnjeg postavljenog pribrojnika
       (zato da NE mora biti argument rekurzivne funkcije). */

int ind_pribr;


    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;


    /* Funkcija za ispis jedne (zadnje) nadjene particije.
       Koristi globalno polje pribr i globalnu varijablu
       ind_pribr za indeks zadnjeg pribrojnika u polju.
         Inicijalizacija prije vanjskog poziva rekurzivne
       funkcije je ind_pribr = -1, kao signal da jos nema
       postavljenih pribrojnika. */

void ispis_particije(void)
{
    int i;

        /* Provjera indeksa zadnjeg pribrojnika (za svaki slucaj). */
    if (ind_pribr < 0) return;

        /* Ispis particije, sigurno je ind_pribr >= 0. */
    printf("  %2d", pribr[0]);
    for (i = 1; i <= ind_pribr; ++i)
        printf(" + %2d", pribr[i]);
    printf("\n");
}

    /* Nadji sve opisane particije broja suma u kojima je
       prvi (sljedeci) pribrojnik barem jednak broju prvi. */

    /* U trenutku poziva funkcije (ulaza u funkciju), globalni
       ind_pribr = indeks zadnjeg (do sada) postavljenog pribrojnika
       u "globalnoj" particiji polaznog broja. Sljedeci pribrojnik
       se postavlja unutar ovog poziva funkcije. */

int particije_3(int suma, int k, int prvi)
{
    int i, broj = 0;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Provjera kraja rekurzije i jesmo li nasli rastav. */
    if (k == 0) {
            /* Svi pribrojnici su vec postavljeni => kraj. */
        if (suma == 0) {

                /* Ispis nadjene particije, sigurno je ind_pribr >= 0. */
            ispis_particije();

                /* Povratak za zavrsenu particiju. */
            return 1;
        }
        else
           return 0;
    }

        /* Povecaj globalni ind_pribr za sljedeci pribrojnik,
           koji se mijenja u sljedecoj petlji po i. */
    ++ind_pribr;

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           uz ogranicenje prvi <= i <= suma:
           - preostala suma je suma - i,
           - broj pribrojnika u preostaloj sumi je jednak k - 1,
           - sljedeci prvi je barem i. */

    for (i = prvi; i <= suma; ++i) {
            /* Postavljanje sljedeceg pribrojnika (i) u polje,
               rekurzivni poziv nakon dodavanja tog pribrojnika */
        pribr[ind_pribr] = i;
        broj += particije_3(suma - i, k - 1, i);
    }

        /* Smanji globalni ind_pribr za povratak. */
    --ind_pribr;

    return broj;
}

    /* Nerekurzivna funkcija za racunanje p_3(n, k).
       Zove rekurzivnu funkciju particije_3,
       s dodatnim argumentom prvi = 1. */

int p_3(int n, int k)
{
        /* Inicijalizacija globalnog indeksa zadnjeg (do sada)
           postavljenog pribrojnika na -1 (jos ih nema). */
    ind_pribr = -1;

        /* Poziv rekurzivne funkcije particije_3, s prvi = 1. */
    return particije_3(n, k, 1);
}

int main(void)
{
    int n, k, max_pribr, broj;

    printf(" Upisi prirodne brojeve n i k: ");
    scanf("%d%d", &n, &k);

        /* Najveci moguci broj pribrojnika max_pribr je bas k. */
    max_pribr = k;

        /* Alokacija polja za pribrojnike. */
    if ((pribr = (int*) malloc(max_pribr * sizeof(int))) == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(1);
    }

    broj_poziva = 0u;

    printf("\n\n");
    printf(" Sve trazene particije broja %d su:\n\n", n);

        /* Poziv funkcije - pise i broji sve particije. */
    broj = p_3(n, k);

        /* Zavrsni ispis broja particija i poziva. */
    printf("\n");
    printf(" Broj particija p_3(%d, %d) = %d\n",
           n, k, broj);
    printf(" Broj poziva funkcije = %u\n",
           broj_poziva);

        /* Dealokacija (oslobadjanje) polja za pribrojnike. */
    free(pribr);

    return 0;
}
