#include <stdio.h>
#include <stdlib.h>

  /* Sve particije - Osnovni problem:
       Svi rastavi zadanog prirodnog broja n u sumu
     nepadajucih (prirodnih) pribrojnika.
     Dozvoljen je i samo jedan pribrojnik.

     Rekurzivno generira (pise, broji i zbraja) sve particije.
     Osnovni rekurzivni algoritam (poput parts_1.c, par0_par.c).

     Ucitava (prirodni) broj n,
     ispisuje sve opisane particije broja n.
     Na kraju, ispisuje broj opisanih particija p(n)
     i broj rekurzivnih poziva funkcije.

     Koristi globalno dinamicki alocirano polje pribr za
     pribrojnike u particiji i globalnu varijablu ind_pribr
     za indeks (trenutno) zadnjeg postavljenog pribrojnika u polju.
  */

    /* Globalni pokazivac na dinamicko polje pribrojnika. */

int *pribr;

    /* Globalna varijabla za indeks zadnjeg postavljenog pribrojnika
       (zato da NE mora biti argument rekurzivne funkcije). */

int ind_pribr;


    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;


    /* Funkcija za ispis jedne (zadnje) nadjene particije.
       Koristi globalno polje pribr i globalnu varijablu
       ind_pribr za indeks zadnjeg pribrojnika u polju.
         Inicijalizacija prije vanjskog poziva rekurzivne
       funkcije je ind_pribr = -1, kao signal da jos nema
       postavljenih pribrojnika. */

void ispis_particije(void)
{
    int i;

        /* Provjera indeksa zadnjeg pribrojnika (za svaki slucaj). */
    if (ind_pribr < 0) return;

        /* Ispis particije, sigurno je ind_pribr >= 0. */
    printf("  %2d", pribr[0]);
    for (i = 1; i <= ind_pribr; ++i)
        printf(" + %2d", pribr[i]);
    printf("\n");
}

    /* Nadji sve opisane particije broja suma u kojima je
       prvi (sljedeci) pribrojnik barem jednak broju prvi. */

    /* U trenutku poziva funkcije (ulaza u funkciju), globalni
       ind_pribr = indeks zadnjeg (do sada) postavljenog pribrojnika
       u "globalnoj" particiji polaznog broja. Sljedeci pribrojnik
       se postavlja unutar ovog poziva funkcije. */

int particije(int suma, int prvi)
{
    int i, broj = 0;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Provjera kraja rekurzije i jesmo li nasli rastav. */
    if (suma == 0) {

            /* Ispis nadjene particije, sigurno je ind_pribr >= 0. */
        ispis_particije();

            /* Povratak za zavrsenu particiju. */
        return 1;
    }

        /* Povecaj globalni ind_pribr za sljedeci pribrojnik,
           koji se mijenja u sljedecoj petlji po i. */
    ++ind_pribr;

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           uz ogranicenje prvi <= i <= suma:
           - preostala suma je suma - i,
           - sljedeci prvi je barem i. */

    for (i = prvi; i <= suma; ++i) {
            /* Postavljanje sljedeceg pribrojnika (i) u polje,
               rekurzivni poziv nakon dodavanja tog pribrojnika */
        pribr[ind_pribr] = i;
        broj += particije(suma - i, i);
    }

        /* Smanji globalni ind_pribr za povratak. */
    --ind_pribr;

    return broj;
}

    /* Nerekurzivna funkcija za racunanje p(n).
       Zove rekurzivnu funkciju particije,
       s dodatnim argumentom prvi = 1. */

int p(int n)
{
        /* Inicijalizacija globalnog indeksa zadnjeg (do sada)
           postavljenog pribrojnika na -1 (jos ih nema). */
    ind_pribr = -1;

        /* Poziv rekurzivne funkcije particije, s prvi = 1. */
    return particije(n, 1);
}

int main(void)
{
    int n, max_pribr, broj;

    printf(" Upisi prirodni broj n: ");
    scanf("%d", &n);

        /* Najveci moguci broj pribrojnika max_pribr je bas n,
           iz rastava n = 1 + 1 + ... + 1 (n jedinica). */
    max_pribr = n;

        /* Alokacija polja za pribrojnike. */
    if ((pribr = (int*) malloc(max_pribr * sizeof(int))) == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(1);
    }

    broj_poziva = 0u;

    printf("\n\n");
    printf(" Sve trazene particije broja %d su:\n\n", n);

        /* Poziv funkcije - pise i broji sve particije. */
    broj = p(n);

        /* Zavrsni ispis broja particija i poziva. */
    printf("\n");
    printf(" Broj particija p(%d) = %d\n",
           n, broj);
    printf(" Broj poziva funkcije = %u\n",
           broj_poziva);

        /* Dealokacija (oslobadjanje) polja za pribrojnike. */
    free(pribr);

    return 0;
}
