#include <stdio.h>

  /* Particije - Varijacija 1:
       Broj rastava zadanog prirodnog broja n u sumu
     strogo rastucih (prirodnih) pribrojnika.
     Dozvoljen je i samo jedan pribrojnik.

     Rekurzivno generira (broji i zbraja) sve particije.
     Efikasni rekurzivni algoritam (poput par_4.c, par_5.c).

     Umjesto citanja broja n,
     ispisuje broj opisanih particija p_1(n),
     za sve brojeve n = 1, ..., 100.
     Ispisuje i broj rekurzivnih poziva funkcije.
  */

    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;


    /* Nadji sve opisane particije broja suma u kojima je
       prvi (sljedeci) pribrojnik barem jednak broju prvi. */

    /* Funkcija OCEKUJE da ulazni argumenti zadovoljavaju SVE
       potrebne uvjete za postojanje (bar) jedne takve particije. */
    /* Ovdje ocekujemo da je prvi <= suma. */

int particije_1(int suma, int prvi)
{
    int i, broj = 0, max_i;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Ovdje OCEKUJEMO da je prvi <= suma (bez provjere).
           Tada ima (bar) jedna takva particija - ona sa samo
           jednim pribrojnikom i = suma.
             Obrada te particije ide na KRAJU, IZA rekurzije,
           zato da generiranje ide u leksikografskom poretku,
           jer ta particija ima najveci pribrojnik i. */

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           uz osnovno ogranicenje prvi <= i < suma (i = suma se
           obradjuje posebno, bez rekurzije):
           - preostala suma je suma - i,
           - sljedeci prvi je barem i + 1.
           Iz uvjeta da je sljedeci prvi <= preostala suma
           dobivamo i + 1 <= suma - i.
           Pripadni test na gornjoj granici petlje po i je
           2*i + 1 <= suma, ili i <= max_i = (suma - 1) / 2. */

    max_i = (suma - 1) / 2;
    for (i = prvi; i <= max_i; ++i)
        broj += particije_1(suma - i, i + 1);

        /* Obrada particije za i = suma.
           Povecaj broj za 1. */
    ++broj;

    return broj;
}

    /* Nerekurzivna funkcija za racunanje p_1(n).
       Zove rekurzivnu funkciju particije_1,
       s dodatnim argumentom prvi = 1. */

int p_1(int n)
{
        /* Provjera uvjeta za vanjski poziv rekurzivne funkcije.
           Ako ocekujemo da je n >= 1, onda je 1 = prvi <= suma = n,
           pa NE treba posebna provjera.
           Ako je dozvoljeno n <= 0, onda provjeravamo taj uvjet
           i odmah vratimo p_1(n) = 0, za n <= 0.
           Radi jednostavnosti, dogovorno uzimamo da je p_1(0) = 0. */
    if (n <= 0) return 0;

        /* Poziv rekurzivne funkcije particije_1, s prvi = 1. */
    return particije_1(n, 1);
}

int main(void)
{
    int n;

        /* Zaglavlje tablice. */
    printf("\n");
    printf("      Broj particija       Broj poziva\n");
    printf(" ---------------------------------------\n");

    for (n = 1; n <= 100; ++n ) {

            /* Inicijalizacija globalnog brojaca poziva. */
        broj_poziva = 0u;

            /* Redak tablice. */
        printf("   p_1(%3d) = %10d    %10u\n",
               n, p_1(n), broj_poziva);
    }

    return 0;
}
