#include <stdio.h>

  /* Particije - Varijacija 2:
       Broj rastava zadanog prirodnog broja n u sumu
     nepadajucih (prirodnih) pribrojnika, s tim da je
     razlika susjednih pribrojnika barem k, gdje je
     k >= 0 zadani cijeli broj.
     Dozvoljen je i samo jedan pribrojnik.

     Rekurzivno generira (broji i zbraja) sve particije.
     Efikasni rekurzivni algoritam (poput par_4.c, par_5.c).
       Varijabla k je globalna (jer se ne mijenja),
     zato da ne mora biti argument rekurzivne funkcije.

     Umjesto citanja brojeva n i k,
     ispisuje broj opisanih particija p_2(n, k),
     za sve brojeve n = 1, ..., 100, uz fiksni k = 3.
     Ispisuje i broj rekurzivnih poziva funkcije.
  */

    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;

    /* Globalna varijabla k za razliku susjednih pribrojnika. */

int k = 3;


    /* Nadji sve opisane particije broja suma u kojima je
       prvi (sljedeci) pribrojnik barem jednak broju prvi. */
    /* Rekurzivna funkcija koristi globalnu varijablu k. */

    /* Funkcija OCEKUJE da ulazni argumenti zadovoljavaju SVE
       potrebne uvjete za postojanje (bar) jedne takve particije. */
    /* Ovdje ocekujemo da je prvi <= suma. */

int particije_2(int suma, int prvi)
{
    int i, broj = 0, max_i;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Ovdje OCEKUJEMO da je prvi <= suma (bez provjere).
           Tada ima (bar) jedna takva particija - ona sa samo
           jednim pribrojnikom i = suma.
             Obrada te particije ide na KRAJU, IZA rekurzije,
           zato da generiranje ide u leksikografskom poretku,
           jer ta particija ima najveci pribrojnik i. */

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           uz osnovno ogranicenje prvi <= i < suma (i = suma se
           obradjuje posebno, bez rekurzije):
           - preostala suma je suma - i,
           - sljedeci prvi je barem i + k.
           Iz uvjeta da je sljedeci prvi <= preostala suma
           dobivamo i + k <= suma - i.
           Pripadni test na gornjoj granici petlje po i je
           2*i + k <= suma, ili i <= max_i = (suma - k) / 2. */

    max_i = (suma - k) / 2;
    for (i = prvi; i <= max_i; ++i)
        broj += particije_2(suma - i, i + k);

        /* Obrada particije za i = suma.
           Povecaj broj za 1. */
    ++broj;

    return broj;
}

    /* Nerekurzivna funkcija za racunanje p_2(n, k).
       Zove rekurzivnu funkciju particije_2,
       s dodatnim argumentom prvi = 1. */

int p_2(int n, int kk)
{
        /* Provjera uvjeta za vanjski poziv rekurzivne funkcije.
           Ako ocekujemo da je n >= 1 i kk >= 0, onda je
           1 = prvi <= suma = n, pa NE treba posebna provjera.
           Ako je dozvoljeno n <= 0 ili kk < 0, onda to provjeravamo
           i odmah vratimo p_2(n, kk) = 0.
           Radi jednostavnosti, uzimamo da je p_2(0, 0) = 0,
           iako je p(0) = 1. */
    if (n <= 0 || kk < 0) return 0;

        /* Postavi globalni k (za rekurziju) na zadani kk. */
    k = kk;

        /* Poziv rekurzivne funkcije particije_2, s prvi = 1. */
    return particije_2(n, 1);
}

int main(void)
{
    int n;

        /* Zaglavlje tablice. */
    printf("\n");
    printf("        Broj particija        Broj poziva\n");
    printf(" ------------------------------------------\n");

    for (n = 1; n <= 100; ++n ) {

            /* Inicijalizacija globalnog brojaca poziva. */
        broj_poziva = 0u;

            /* Redak tablice. */
        printf("   p_2(%3d, %d) = %10d    %10u\n",
               n, k, p_2(n, k), broj_poziva);
    }

    return 0;
}
