#include <stdio.h>

  /* Ucitava n i pise broj particija broja n,
     tj. broj rastava broja n u sumu nepadajucih
     prirodnih brojeva.
     Rekurzivno broji i zbraja sve generirane particije.

     Efikasni algoritam, BEZ nepotrebnih rekurzivnih poziva,
     tj. broj poziva = broj particija.
  */

    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;

    /* Nadji sve particije broja suma u kojima je prvi (sljedeci)
       pribrojnik barem jednak prvi. */

int particije(int suma, int prvi)
{
    int i, broj = 0;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Uociti da funkcija particije vraca 0 ako i samo ako je
           prvi > suma (pogledati par_3.c). Upravo takve NEPOTREBNE
           pozive (koji ne daju nove particije) zelimo izbjeci.

           Drugim rijecima, ako zelimo da svaki poziv funkcije
           generira bar jednu novu particiju, onda na ulazu MORA
           vrijediti prvi <= suma.

           Tada sigurno postoji (bar) jedna nova particija - ona s
           jednim dodatnim pribrojnikom i = suma. Ta particija je
           sigurno zavrsena, jer je preostala suma za rastav
           (nakon dodavanja tog pribrojnika) jednaka suma - i = 0.
           Onda odmah mozemo povecati broj za 1.

           Medjutim, ako ocekujemo prvi >= suma na ULAZU, onda moramo
           osigurati da to vrijedi i za sve rekurzivne pozive koje
           radimo u nastavku. Ti pozivi imaju oblik
              ... particije(suma - i, i);
           Dakle, kod tih poziva MORA vrijediti suma - i >= i.

           I to je jednostavno napraviti - tako da modificiramo
           GORNJU granicu petlje. Umjesto ranijeg uvjeta i <= suma
           (ili i < suma), treba staviti i <= suma - i.
           Uociti da je preostala suma - i sigurno pozitivna (jer
           je i < suma, zbog i >= 1), tj. pripadna particija nije
           zavrsena (fale joj pribrojnici). Stoga rekurzivni pozivi
           imaju smisla (da dodaju preostale potrebne pribrojnike).

           ZAVRSNA primjedba: funkcija OCEKUJE da ulazni argumenti
           zadovoljavaju SVE potrebne uvjete za postojanje (bar) jedne
           particije. Ovdje je samo potrebno da je prvi <= suma.

           Test toga je potreban samo za prvi poziv (onaj izvana),
           a svi ostali rekurzivni pozivi ne trebaju takav test.
           Ako smo sigurni da je i vanjski poziv korektan, onda
           ne treba nista dodatno provjeravati!
        */

        /* Ovdje OCEKUJEMO da je prvi <= suma (bez provjere).
           Tada ima (bar) jedna takva particija, ona za i = suma.
           Odmah povecaj broj za 1. */
    ++broj;

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           za i <= suma - i < suma.
           Test na GORNJOJ granici i <= suma - i osigurava da u
           nastavku rekurzije vrijedi prvi <= suma, tj. da postoje
           takve particije u kojima je sljedeci pribrojnik barem i. */
    for (i = prvi; i <= suma - i; ++i)
        broj += particije(suma - i, i);

    return broj;
}

int main(void)
{
    int n;

    printf(" Upisi prirodni broj n: ");
    scanf("%d", &n);

    broj_poziva = 0u;

        /* Ovdje je n >= 1, tj. suma >= prvi, pa NE treba
           provjera argumenata za vanjski (prvi) poziv. */

    printf("\n Broj particija p(%d) = %d\n",
           n, particije(n, 1) );

    printf("\n Broj poziva funkcije = %u\n",
           broj_poziva);

    return 0;
}
