#include <stdio.h>

  /* Ucitava n i pise broj particija broja n,
     tj. broj rastava broja n u sumu nepadajucih
     prirodnih brojeva.
     Rekurzivno broji i zbraja sve generirane particije.

     Efikasni algoritam, BEZ nepotrebnih rekurzivnih poziva,
     tj. broj poziva = broj particija.

     Modificirana varijanta, prilagodjena varijacijama problema.
  */

    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva;

    /* Nadji sve particije broja suma u kojima je prvi (sljedeci)
       pribrojnik barem jednak prvi. */

    /* Funkcija OCEKUJE da ulazni argumenti zadovoljavaju SVE
       potrebne uvjete za postojanje (bar) jedne particije.

       Test toga je potreban samo za prvi poziv (onaj izvana),
       a svi ostali rekurzivni pozivi ne trebaju takav test.
       Ako smo sigurni da je i vanjski poziv korektan, onda
       ne treba nista dodatno provjeravati! */

int particije(int suma, int prvi)
{
    int i, broj = 0;
    int preost_suma, sljed_prvi;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Ovdje OCEKUJEMO da je prvi <= suma (bez provjere).
           Tada ima (bar) jedna takva particija - ona sa samo
           jednim pribrojnikom i = suma.
           Odmah povecaj broj za 1. */
    ++broj;

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i.
           Granica za petlju je i < suma (i = suma je vec obradjen),
           a test uvjeta za prekid petlje i rekurzivnih poziva
           je ugradjen unutar petlje. */
    for (i = prvi; i < suma; ++i) {

            /* Vrijednost za nastavak rekurzije. */
        preost_suma = suma - i;
        sljed_prvi = i;

            /* Test uvjeta za prekid petlje (daljnji rekurzivni
               pozivi ne generiraju nove particije). */
        if (sljed_prvi > preost_suma) break;

        broj += particije(preost_suma, sljed_prvi);
    }

    return broj;
}

int main(void)
{
    int n;

    printf(" Upisi prirodni broj n: ");
    scanf("%d", &n);

    broj_poziva = 0u;

        /* Ovdje je n >= 1, tj. suma >= prvi, pa NE treba
           provjera argumenata za vanjski (prvi) poziv. */

    printf("\n Broj particija p(%d) = %d\n",
           n, particije(n, 1) );

    printf("\n Broj poziva funkcije = %u\n",
           broj_poziva);

    return 0;
}
