#include <stdio.h>
#include <string.h>
#include <stdlib.h>

  /* Sortiranje rjecnika. */

  /* Varijacija 1: polja w i p se dinamicki (re)alociraju.
     Poboljsana varijanta, koja alocira i postavlja polje p
     samo jednom, na samom kraju ucitavanja, iz polja w.

     Sortiranje zamjenama pokazivaca u p - izbor ekstrema.

     Funkcija unos vraca status ucitavanja rijeci,
     a broj ucitanih rijeci je globalna varijabla.

     Za ucitavanje koristimo da je poznata maksimalna
     moguca duljina rijeci MAXDULJ = 80.
     To je dovoljno za sve prakticne potrebe.
     Nema smisla raditi dinamicko alociranje "znak-po-znak" za
     citanje stringa nepoznate duljine (v. vjezbe, poglavlje 5.2). */

    /* Funkcija my_gets za citanje stringa. */
#include "my_gets.c"


    /* Simbolicka konstanta:
       MAXDULJ = max. duljina pojedine rijeci,
       ukjucivo i nul-znak na kraju stringa. */

#define MAXDULJ 80

    /* Globalno dinamicko polje znakova za cijeli rjecnik.
       Globalna duljina dulj_w sluzi samo za kontrolu. */

char *w = NULL;
unsigned dulj_w = 0u;

    /* Globalno dinamicko polje pokazivaca
       na pojedine rijeci - stringove. */

char **p = NULL;

    /* Globalni stvarni broj rijeci u rjecniku = duljina polja p. */

int broj_rijeci = 0;


    /* Ucitava rijeci i vraca status ucitavanja (0 = nema greske).
       Kraj ucitavanja je prazna rijec.
       Broj ucitanih rijeci broj_rijeci je globalna varijabla. */

int unos(void)
{
    char q[MAXDULJ] = "";    /* Buffer za citanje stringa. */
    char *temp_w, *poc_rijeci;
    int status = 0, broj = 0, dulj, i;
    unsigned novi_dulj_w;

    while (1) {

            /* Citanje stringa u buffer i test greske. */
        if (my_gets(q, MAXDULJ) == NULL) {
            status = -1;
            break;
        }

        dulj = strlen(q);

            /* Test na praznu rijec - kraj ucitavanja. */
        if (dulj == 0) break;

            /* Imamo nepraznu novu rijec. */

            /* Realociraj w. */
            /* Ako realokacija nije uspjela, vrati do tada
               ucitani w i broj rijeci, i postavi p nakon petlje. */
        novi_dulj_w = dulj_w + dulj + 1;
        temp_w = (char *) realloc(w, novi_dulj_w * sizeof(char));
        if (temp_w == NULL) {
            status = 1;
            break;
        }

            /* Realokacija je uspjela, postavi (mozda) novi w. */
        w = temp_w;

            /* Kopiraj novu rijec na kraj polja w. */
        poc_rijeci = w + dulj_w;
        strcpy(poc_rijeci, q);
        dulj_w = novi_dulj_w;

            /* Kontrolni ispis. */
        printf(" rijec[%d] = %s\n", broj, poc_rijeci);

            /* Na kraju, povecaj broj rijeci. */
        ++broj;
    }

        /* Alociraj polje pokazivaca p.
           Test na broj > 0 nije potreban. */
    p = (char **) malloc(broj * sizeof(char *));
    if (p == NULL) {
        broj = 0;    /* Ne mozemo spremiti polje p. */
        status = 2;
    }
    else {

            /* Postavi sve pokazivace u polju p. */

        temp_w = w;
        for (i = 0; i < broj; ++i) {
            p[i] = temp_w;
            temp_w = temp_w + strlen(temp_w) + 1;
        }
    }

        /* Postavi globalni broj_rijeci. */
    broj_rijeci = broj;
    return status;
}

    /* Sortiranje rjecnika izborom ekstrema.
       Dovodimo najmanji element na pocetak.

       Rjecnik je zadan poljem pokazivaca na rijeci
       (element polja je pokazivac na znak-string).
       Koristimo samo zamjene pokazivaca. */

void sort(char *p[], int n)
{
    int i, j, ind_min;
    char *temp;

    for (i = 0; i < n - 1; ++i) {
        ind_min = i;
        for (j = i + 1; j < n; ++j)
            if (strcmp(p[j], p[ind_min]) < 0)
                ind_min = j;
        if (i != ind_min) {
            temp = p[i];
            p[i] = p[ind_min];
            p[ind_min] = temp;
        }
    }

    return;
}

    /* Ispisuje sve rijeci u rjecniku. */

void ispis(char *p[])
{
    int i;

    for (i = 0; i < broj_rijeci; ++i)
        puts(p[i]);

    return;
}

    /* Glavni program. */

int main(void)
{
    int status;

    status = unos();
    if (status == -1)
        printf("Greska: EOF u citanju rijeci.\n");
    else if (status == 1)
        printf("Greska: Realokacija za w nije uspjela.\n");
    else if (status == 2)
        printf("Greska: Alokacija za p nije uspjela.\n");

        /* Namjerno, u gornjem if, na kraju nema else:
           bez obzira na eventualnu gresku, sortiramo
           sve uspjesno ucitane rijeci. */

    printf("Broj rijeci = %d\n", broj_rijeci);
    printf("\nPolazni rjecnik:\n");
    ispis(p);

        /* Sortiranje polja pokazivaca p. */
    sort(p, broj_rijeci);

    printf("\nSortirani rjecnik:\n");
    ispis(p);

        /* Oslobodi svu zauzetu memoriju.
           Zbog stalne realokacije polja w,
           ovo radi i kad dodje do greske u citanju. */
    free(w);
    free(p);

    return 0;
}
