#include <stdio.h>
#include <stdlib.h>

  /* Deklaracije za vezanu listu cijelih brojeva
     i osnovne funkcije za rad s takvim listama.
  */

/* ======================================================================== */

    /* Tip za pokazivac na element liste. */

typedef struct _element *lista;

    /* Tip za element liste. */

typedef struct _element {
    int broj;            /* Sadrzaj je broj. */
    lista sljed;         /* Pokazivac na sljedeci element u listi. */
} element;

/* ======================================================================== */

lista kreiraj_novi(int broj)
{
    lista novi = NULL;
/*
    if ((novi = (lista) malloc(sizeof(element))) == NULL)
*/
    novi = (lista) malloc(sizeof(element));
    if (novi == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(EXIT_FAILURE);    /* exit(1); */
    }

    novi->broj = broj;
    novi->sljed = NULL;

    return novi;
}

/* ======================================================================== */

lista obrisi_element(lista stari)
{
    free(stari);

    return NULL;    /* Umjesto stari = NULL; */
}

/* ======================================================================== */

lista ubaci_na_pocetak(lista prvi, lista novi)
{
        /* Ne provjerava novi != NULL. */

    novi->sljed = prvi;
    prvi = novi;

    return prvi;
}

/* ======================================================================== */

lista obrisi_prvog(lista prvi)
{
    lista pom;

    if (prvi != NULL) {
        pom = prvi;
        prvi = prvi->sljed;
        free(pom);
            /* Ne treba pom = NULL; */
    }

    return prvi;
}

/* ======================================================================== */

lista kreiraj_sprijeda(lista prvi, int broj)
{
    lista novi = NULL;
/*
    if ((novi = (lista) malloc(sizeof(element))) == NULL)
*/
    novi = (lista) malloc(sizeof(element));
    if (novi == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(EXIT_FAILURE);    /* exit(1); */
    }

    novi->broj = broj;
    novi->sljed = prvi;

        /* Ne treba prvi = novi, vec samo ovo: */
    return novi;
}

/* ======================================================================== */

lista obrisi_listu(lista prvi)
{
    lista pom;
/*
    while (prvi != NULL)
        prvi = obrisi_prvog(prvi);
*/
    while (prvi != NULL) {
        pom = prvi;
        prvi = prvi->sljed;
        free(pom);
    }

    return NULL;    /* <=> return prvi; */
}

/* ======================================================================== */

int broj_elemenata(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed)
        ++brojac;

    return brojac;
}

/* ======================================================================== */

void ispisi_listu(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed) {
        printf(" Element %2d, broj = %2d\n",
               ++brojac, pom->broj);
    }

    return;
}

/* ======================================================================== */

void ispisi_listu_2(lista prvi)
{
    lista pom;
    int brojac = 0;

    if (prvi == NULL)
        printf(" Lista je prazna!\n");
    else
        for (pom = prvi; pom != NULL; pom = pom->sljed) {
            printf(" Element %2d, broj = %2d\n",
                   ++brojac, pom->broj);
        }

    return;
}

/* ======================================================================== */

void ispisi_listu_short(lista prvi)
{
    lista pom;

    for (pom = prvi; pom != NULL; pom = pom->sljed) {
        printf(" %d ->", pom->broj);
    }
    printf(" NULL\n");

    return;
}

/* ======================================================================== */

lista trazi_broj(lista prvi, int broj)
{
    lista pom = prvi;

    while (pom != NULL && pom->broj != broj)
        pom = pom->sljed;

    return pom;
}

/* ======================================================================== */

lista trazi_zadnji(lista prvi)
{
    lista pom;

    if (prvi == NULL) return NULL;

    for (pom = prvi; pom->sljed != NULL; pom = pom->sljed);

    return pom;
}

/* ======================================================================== */

lista ubaci_iza(lista prvi, lista preth, lista novi)
{
        /* Ne provjerava novi != NULL. */

        /* Ako je preth == NULL, ubacujemo na pocetak. */
    if (preth == NULL) {
        novi->sljed = prvi;
        prvi = novi;
    }
    else {
        novi->sljed = preth->sljed;
        preth->sljed = novi;
    }

    return prvi;
}

/* ======================================================================== */

lista obrisi_iza(lista prvi, lista preth)
{
    lista pom;

        /* Ako je preth == NULL, brisemo prvi element. */
    if (preth == NULL) {
        pom = prvi;
        prvi = prvi->sljed;
    }
    else {
        pom = preth->sljed;
        preth->sljed = pom->sljed;
    }
    free(pom);

    return prvi;
}

/* ======================================================================== */

lista obrisi_iza_pazljivo(lista prvi, lista preth)
{
    lista pom;

        /* Provjera prazne liste. */
    if (prvi == NULL) return NULL;

        /* Ako je preth == NULL, brisemo prvi element. */
    if (preth == NULL) {
        pom = prvi;
        prvi = prvi->sljed;
    }
    else {
        pom = preth->sljed;
            /* Ako preth nije zadnji, izbaci sljedeceg. */
        if (pom != NULL)
            preth->sljed = pom->sljed;
    }
    free(pom);    /* Ovo radi i za pom == NULL. */

    return prvi;
}

/* ======================================================================== */

lista ubaci_na_kraj_lose(lista prvi, lista novi)
{
        /* Ne provjerava novi != NULL. */

    lista zadnji;

        /* Moze i samo else blok. */
    if (prvi == NULL)
        prvi = ubaci_na_pocetak(prvi, novi);
    else {
        zadnji = trazi_zadnji(prvi);
        prvi = ubaci_iza(prvi, zadnji, novi);
    }

    return prvi;
}

/* ======================================================================== */

lista ubaci_na_kraj(lista prvi, lista *p_zadnji, lista novi)
{
        /* Ne provjerava novi != NULL. */
        /* Vraca zadnji kroz varijabilni argument - pokazivac p_zadnji. */

    lista zadnji = *p_zadnji;

        /* Moze: prvi == NULL || zadnji == NULL. */
    if (prvi == NULL)
        prvi = novi;
    else    /* Ocekujemo zadnji->sljed == NULL. */
        zadnji->sljed = novi;
    novi->sljed = NULL;

/* Ne treba:
    zadnji = novi; *p_zadnji = zadnji;
*/
    *p_zadnji = novi;    /* Vrati novi zadnji! */

    return prvi;
}

/* ======================================================================== */

lista kreiraj_straga(lista prvi, lista *p_zadnji, int broj)
{
        /* Vraca zadnji kroz varijabilni argument - pokazivac p_zadnji. */

    lista zadnji = *p_zadnji;
    lista novi = NULL;
/*
    if ((novi = (lista) malloc(sizeof(element))) == NULL)
*/
    novi = (lista) malloc(sizeof(element));
    if (novi == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(EXIT_FAILURE);    /* exit(1); */
    }

    novi->broj = broj;
    novi->sljed = NULL;

    if (prvi == NULL)
        prvi = novi;
    else
        zadnji->sljed = novi;

/* Ne treba:
    zadnji = novi; *p_zadnji = zadnji;
*/
    *p_zadnji = novi;    /* Vrati novi zadnji! */

    return prvi;
}

/* ======================================================================== */

lista sortirano_ubaci_0(lista prvi, lista novi)
{
        /* Insertion sort za jedan element. */
        /* Ne provjerava novi != NULL. */

    lista preth, pom;

/*    Ovo ispod ne treba i jos moze NE RADITI,
      ako nismo sigurni da je novi->sljed == NULL.

    if (prvi == NULL) return novi;
*/

    pom = prvi;
/*    NE treba inicijalizacija preth = NULL. Razlog:
      Kod ubacivanja na pocetak liste, koristimo
      uvjet (pom == prvi), a ne (preth == NULL). */

        /* Nadji trazeni element, ako ga ima. */
    while (pom != NULL && pom->broj < novi->broj) {
        preth = pom;
        pom = preth->sljed;    /* <=> pom = pom->sljed; */
    }

/*    Ispod je ubaci_iza(prvi, preth, novi) s
      promjenom uvjeta. To radi i za prvi == NULL.
*/
    if (pom == prvi) {    /* preth ne treba. */
        novi->sljed = prvi;
        prvi = novi;
    }
    else {    /* Tu je pom == preth->sljed. */
        novi->sljed = preth->sljed;
        preth->sljed = novi;
    }

    return prvi;
}

/* ======================================================================== */

lista sortirano_ubaci(lista prvi, lista novi)
{
        /* Insertion sort za jedan element. */
        /* Ne provjerava novi != NULL. */

        /* Inicijalizacija pom na prvi. NE treba
           inicijalizacija preth na NULL. */
    lista preth, pom = prvi;

        /* Nadji trazeni element, ako ga ima. */
    while (pom != NULL && pom->broj < novi->broj) {
        preth = pom;
        pom = preth->sljed;    /* <=> pom = pom->sljed; */
    }

/*    Neovisno o pom == prvi ili ne, pom uvijek
      pokazuje na listu sljedbenika novog elementa.
*/
        /* Objesi pom iza novog. */
    novi->sljed = pom;

        /* Sad pitamo kamo ce novi. */
    if (pom == prvi)
        prvi = novi;
    else    /* Tu je pom == preth->sljed. */
        preth->sljed = novi;

    return prvi;
}

/* ======================================================================== */

lista spoji_dvije(lista prvi_1, lista prvi_2)
{
    lista pom;

        /* Ako je prva lista prazna, rezultat je druga lista. */
    if (prvi_1 == NULL) return prvi_2;

        /* Nadji zadnjeg u prvoj i spoji drugu listu na njega. */
    for (pom = prvi_1; pom->sljed != NULL; pom = pom->sljed);
    pom->sljed = prvi_2;

    return prvi_1;
}

/* ======================================================================== */
