#include <stdio.h>
#include <stdlib.h>

  /* Preuredjenje vezane liste - prvo parni, onda neparni.
     CUVA relativni poredak parnih, odnosno, neparnih elemenata.

     Algoritam 1a:
     - Rastav polazne liste u dvije zasebne liste, spajanje na kraju.
     - Obje liste su stalno uzemljene.
     NIJE efikasan, zbog stalnog uzemljenja (azuriranje pokazivaca)!
     Jedina usteda (obzirom na Algoritam 0) je kod spajanja - nema
     nepotrebnog trazenja zadnjeg elementa u listi parnih.
  */

/* ======================================================================== */

    /* Tip za pokazivac na element liste. */

typedef struct _element *lista;

    /* Tip za element liste. */

typedef struct _element {
    int broj;            /* Sadrzaj je broj. */
    lista sljed;         /* Pokazivac na sljedeci element u listi. */
} element;

/* ======================================================================== */

    /* Preuredjuje listu tako da na pocetku liste budu svi parni,
       a zatim dolaze svi neparni elementi iz polazne liste.
       Relativni poredak parnih, odnosno, neparnih elemenata
       je ISTI kao u polaznoj listi. */

    /* Algoritam 1a:
         Rastavi polaznu listu u dvije zasebne liste (parni, neparni),
       a zatim ih spoji (konkatencija).
       Obje liste su stalno uzemljene (neefikasno).

         Za CUVANJE relativnog poretka, radimo ubacivanje na KRAJ
       odgovarajuce liste. Zato pamtimo prvi i zadnji element u
       obje liste (parni, neparni). Dodatno, to bitno ubrzava spajanje
       (konkatenaciju) - ne treba traziti zadnjeg u listi parnih.

         Unutar petlje za obradu, sljedbenik zadnjeg se uvijek
       postavlja na NULL.
    */

lista preuredi_par_nepar_1a(lista prvi)
{
        /* Inicijalizacije liste parnih i liste neparnih na praznu listu.
           Za obje liste pamtimo pokazivace na prvi i zadnji element.
           Pokazivace na prvi element inicijaliziramo na NULL.
           Pokazivace na zadnji element NE treba inicijalizirati na NULL. */
    lista prvi_parni = NULL, zadnji_parni;
    lista prvi_neparni = NULL, zadnji_neparni;

        /* Pokazivac na radni element (onaj kojeg obradjujemo). */
    lista radni;

        /* Petlja za prolaz po polaznoj listi.
           Napreduje za po jedan radni element. */

    while (prvi != NULL) {

            /* Izbaci prvi preostali element (= radni) i makni pocetak. */
        radni = prvi;
        prvi = prvi->sljed;

            /* Sad obradjujemo izbaceni radni element, tj. ubacujemo
               ga na KRAJ odgovarajuce liste, ovisno o parnosti broja. */

            /* Odmah uzemljimo radni, tj. postavimo mu sljedbenika na NULL.
               To osigurava da odgovarajuca lista ima korektan zavrsetak,
               neovisno o tome u KOJU listu ga ubacimo na kraj. Onda NE
               moramo pisati uzemljenje zadnjeg za svaku listu posebno
               (zadnji_neki->sljed = NULL, neki = parni ili neparni). */
        radni->sljed = NULL;

            /* Ovo ispod je ekvivalentno pozivu funkcije ubaci_na_kraj
               za odgovarajucu listu (parni, odnosno, neparni). */

        if (radni->broj % 2 == 0) {    /* Parni. */

                /* Ubaci ga na kraj liste parnih. */
            if (prvi_parni == NULL)
                prvi_parni = radni;
            else    /* Tu je zadnji_parni->sljed == NULL. */
                zadnji_parni->sljed = radni;
            zadnji_parni = radni;    /* Vec je radni->sljed == NULL! */
        }
        else {    /* Neparni. */

                /* Ubaci ga na kraj liste neparnih. */
            if (prvi_neparni == NULL)
                prvi_neparni = radni;
            else    /* Tu je zadnji_neparni->sljed == NULL. */
                zadnji_neparni->sljed = radni;
            zadnji_neparni = radni;    /* Vec je radni->sljed == NULL! */
        }
    }

        /* Spajanje (konkatenacija) liste parnih i liste neparnih,
           BEZ poziva funkcije spoji_dvije(prvi_parni, prvi_neparni).
           Obje liste su uzemljene, pa NE treba paziti na uzemljenje
           spojene liste.
           - Za spajanje koristimo vec postavljeni zadnji_parni,
             ako lista parnih NIJE prazna.
           - Postavimo i vratimo pokazivac na pocetak spojene liste,
             ovisno o tome je li lista parnih prazna ili ne. */

    if (prvi_parni == NULL)
            /* Nema parnih - nema spajanja. Rezultat je
               lista neparnih (smije i ona biti prazna). */
        return prvi_neparni;
    else {
            /* Ima parnih - spoji listu neparnih iza njih.
               Lista neparnih smije biti prazna. */
        zadnji_parni->sljed = prvi_neparni;
        return prvi_parni;
    }
}

/* ======================================================================== */

    /* Ispod su funkcije za osnovne operacije s listama. */

/* ======================================================================== */

lista kreiraj_straga(lista prvi, lista *p_zadnji, int broj)
{
        /* Vraca zadnji kroz varijabilni argument - pokazivac p_zadnji. */

    lista zadnji = *p_zadnji;
    lista novi = NULL;
/*
    if ((novi = (lista) malloc(sizeof(element))) == NULL)
*/
    novi = (lista) malloc(sizeof(element));
    if (novi == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(EXIT_FAILURE);    /* exit(1); */
    }

    novi->broj = broj;
    novi->sljed = NULL;

    if (prvi == NULL)
        prvi = novi;
    else
        zadnji->sljed = novi;

/* Ne treba:
    zadnji = novi; *p_zadnji = zadnji;
*/
    *p_zadnji = novi;    /* Vrati novi zadnji! */

    return prvi;
}

/* ======================================================================== */

lista obrisi_listu(lista prvi)
{
    lista pom;
/*
    while (prvi != NULL)
        prvi = obrisi_prvog(prvi);
*/
    while (prvi != NULL) {
        pom = prvi;
        prvi = prvi->sljed;
        free(pom);
    }

    return NULL;    /* <=> return prvi; */
}

/* ======================================================================== */

int broj_elemenata(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed)
        ++brojac;

    return brojac;
}

/* ======================================================================== */

void ispisi_listu(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed) {
        printf(" Element %2d, broj = %2d\n",
               ++brojac, pom->broj);
    }

    return;
}

/* ======================================================================== */

int main(void)
{
    lista prvi = NULL;    /* Prazna lista! */
    lista zadnji;
    int broj;

        /* Punjenje liste straga. */

        /* Citanje niza brojeva za listu, do broj = 0.
           To je oznaka kraj niza, ali nije clan niza. */
        /* Poredak brojeva u listi je isti kao redoslijed
           citanja, jer dodajemo na kraj liste. */

    printf("Upisite niz cijelih brojeva (0 = kraj).\n");
    do {
        printf("Sljedeci broj:\n");
        scanf("%d", &broj);
        if (broj != 0)
            prvi = kreiraj_straga(prvi, &zadnji, broj);
    } while (broj != 0);

        /* Ispis broja elemenata i cijele liste. */

    printf("\n");
    printf(" Polazna lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);


        /* Preuredjenje liste - prvo parni, onda neparni. */

    prvi = preuredi_par_nepar_1a(prvi);

        /* Preuredjena lista:
           ispis broja elemenata i cijele liste. */

    printf("\n");
    printf(" Preuredjena lista (parni, neparni):\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);

        /* Obrisi cijelu listu. */

    prvi = obrisi_listu(prvi);

    return 0;
}
