#include <stdio.h>
#include <stdlib.h>

  /* Preuredjenje vezane liste - prvo parni, onda neparni.
     CUVA relativni poredak parnih, odnosno, neparnih elemenata.

     Algoritam 2a:
     - Rastav polazne liste s jednom zasebnom listom, spajanje na kraju.
     - Obje liste su stalno uzemljene.
     NIJE efikasan, zbog stalnog uzemljenja (azuriranje pokazivaca)!
  */

/* ======================================================================== */

    /* Tip za pokazivac na element liste. */

typedef struct _element *lista;

    /* Tip za element liste. */

typedef struct _element {
    int broj;            /* Sadrzaj je broj. */
    lista sljed;         /* Pokazivac na sljedeci element u listi. */
} element;

/* ======================================================================== */

    /* Preuredjuje listu tako da na pocetku liste budu svi parni,
       a zatim dolaze svi neparni elementi iz polazne liste.
       Relativni poredak parnih, odnosno, neparnih elemenata
       je ISTI kao u polaznoj listi. */

    /* Algoritam 2a:
         Rastavi polaznu listu tako da jedna vrsta elemenata (parni)
       ostaje u listi (na pocetku, u prednjem = sredjenom dijelu), a
       druga vrsta elemenata (neparni) se izbacuje i prebacuje u (jednu)
       zasebnu listu. Na kraju, spoji te dvije liste (konkatencija).
       Obje liste su stalno uzemljene (neefikasno).

         Drugim rijecima, umjesto gradjenja zasebne liste parnih,
       ta lista se gradi u prednjem (sredjenom) dijelu "radne" liste.
       IZBACIVANJE neparnih elemenata (uz korektno vezanje preostalih
       parnih) osigurava CUVANJE relativnog poretka parnih!

         Za CUVANJE relativnog poretka, radimo ubacivanje na KRAJ
       odgovarajuce liste. Zato pamtimo prvi i zadnji element u
       obje liste (parni, neparni). Dodatno, to bitno ubrzava spajanje
       (konkatenaciju) - ne treba traziti zadnjeg u listi parnih.
       Zato sto se lista parnih gradi u radnoj listi, zadnji sredjeni
       element je upravo "prethodnik" elementa kojeg obradjujemo,
       tj. prvi_parni <=> prvi, zadnji_parni <=> preth.

         Dakle, algoritam se svodi na izbacivanje neparnih iz polazne
       liste i gradjenje zasebne liste neparnih, dodavanjem na kraj.

         Unutar petlje za obradu, sljedbenik zadnjeg u zasebnoj listi
       (neparni) se uvijek postavlja na NULL.
    */

lista preuredi_par_nepar_2a(lista prvi)
{
        /* Inicijalizacija liste neparnih na praznu listu.
           Za listu pamtimo pokazivace na prvi i zadnji element.
           Pokazivac na prvi element inicijaliziramo na NULL.
           Pokazivac na zadnji element NE treba inicijalizirati na NULL. */
    lista prvi_neparni = NULL, zadnji_neparni;

        /* Pokazivac na radni element (onaj kojeg obradjujemo).
           Inicijaliziramo ga na prvi. */
    lista radni = prvi;

        /* Pokazivac na element ispred onog kojeg obradjujemo.
           NE treba ga inicijalizirati na NULL.
           Za listu parnih, prvi_parni = prvi, zadnji_parni = preth. */
    lista preth;

        /* Pomocni pokazivac na izbaceni neparni element. */
    lista izbacen;

        /* Petlja za prolaz po polaznoj listi.
           Napreduje za po jedan radni element. */

    while (radni != NULL) {

            /* Pokazivac radni pokazuje na element kojeg obradjujemo,
               ovisno o parnosti broja (ostavi ili izbaci/ubaci). */

        if (radni->broj % 2 == 0) {    /* Parni. */

                /* Nema izbacivanja (ostaje u listi). Makni preth i radni
                   za jedan element unaprijed (preth je zadnji parni,
                   a radni je sljedeci kojeg treba obraditi). */
            preth = radni;
            radni = preth->sljed;
        }
        else {    /* Neparni. */

                /* Zapamti pokazivac na neparni element kojeg treba
                   izbaciti. Potreban je za ubacivanje u listu neparnih.
                   Razlog: kod izbacivanja, odmah postavljamo (novi) radni
                   na sljedeci element kojeg treba obraditi. */
            izbacen = radni;

                /* Izbaci element iz radne liste, postavi novi radni.
                   Pokazivac preth se NE mijenja, zbog izbacivanja.
                   Ovo azurira pokazivace prvi i preth (= zadnji)
                   u sredjenom (parnom) dijelu radne liste. */
            if (radni == prvi) {    /* preth ne treba. */
                prvi = prvi->sljed;
                radni = prvi;
            }
            else {    /* Tu je radni == preth->sljed. */
                preth->sljed = radni->sljed;
                radni = preth->sljed;    /* <=> radni = radni->sljed; */
            }

                /* Ubaci izbacenog na kraj liste neparnih. */
            if (prvi_neparni == NULL)
                prvi_neparni = izbacen;
            else    /* Tu je zadnji_neparni->sljed == NULL. */
                zadnji_neparni->sljed = izbacen;

                /* Makni zadnji neparni i uzemlji ga. */
            zadnji_neparni = izbacen;
            zadnji_neparni->sljed = NULL;
        }
    }

        /* Spajanje (konkatenacija) radne liste (parne) i liste neparnih.
           Obje liste su uzemljene, pa NE treba paziti na uzemljenje
           spojene liste.
           - Za spajanje koristimo vec postavljeni zadnji_parni = preth,
             ako lista parnih NIJE prazna (prvi_parni = prvi).
           - Postavimo i vratimo pokazivac na pocetak spojene liste,
             ovisno o tome je li lista parnih prazna ili ne. */

    if (prvi == NULL)
            /* Nema parnih - nema spajanja. Rezultat je
               lista neparnih (smije i ona biti prazna). */
        return prvi_neparni;
    else {
            /* Ima parnih - spoji listu neparnih iza njih.
               Lista neparnih smije biti prazna. */
        preth->sljed = prvi_neparni;
        return prvi;
    }
}

/* ======================================================================== */

    /* Ispod su funkcije za osnovne operacije s listama. */

/* ======================================================================== */

lista kreiraj_straga(lista prvi, lista *p_zadnji, int broj)
{
        /* Vraca zadnji kroz varijabilni argument - pokazivac p_zadnji. */

    lista zadnji = *p_zadnji;
    lista novi = NULL;
/*
    if ((novi = (lista) malloc(sizeof(element))) == NULL)
*/
    novi = (lista) malloc(sizeof(element));
    if (novi == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(EXIT_FAILURE);    /* exit(1); */
    }

    novi->broj = broj;
    novi->sljed = NULL;

    if (prvi == NULL)
        prvi = novi;
    else
        zadnji->sljed = novi;

/* Ne treba:
    zadnji = novi; *p_zadnji = zadnji;
*/
    *p_zadnji = novi;    /* Vrati novi zadnji! */

    return prvi;
}

/* ======================================================================== */

lista obrisi_listu(lista prvi)
{
    lista pom;
/*
    while (prvi != NULL)
        prvi = obrisi_prvog(prvi);
*/
    while (prvi != NULL) {
        pom = prvi;
        prvi = prvi->sljed;
        free(pom);
    }

    return NULL;    /* <=> return prvi; */
}

/* ======================================================================== */

int broj_elemenata(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed)
        ++brojac;

    return brojac;
}

/* ======================================================================== */

void ispisi_listu(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed) {
        printf(" Element %2d, broj = %2d\n",
               ++brojac, pom->broj);
    }

    return;
}

/* ======================================================================== */

int main(void)
{
    lista prvi = NULL;    /* Prazna lista! */
    lista zadnji;
    int broj;

        /* Punjenje liste straga. */

        /* Citanje niza brojeva za listu, do broj = 0.
           To je oznaka kraj niza, ali nije clan niza. */
        /* Poredak brojeva u listi je isti kao redoslijed
           citanja, jer dodajemo na kraj liste. */

    printf("Upisite niz cijelih brojeva (0 = kraj).\n");
    do {
        printf("Sljedeci broj:\n");
        scanf("%d", &broj);
        if (broj != 0)
            prvi = kreiraj_straga(prvi, &zadnji, broj);
    } while (broj != 0);

        /* Ispis broja elemenata i cijele liste. */

    printf("\n");
    printf(" Polazna lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);


        /* Preuredjenje liste - prvo parni, onda neparni. */

    prvi = preuredi_par_nepar_2a(prvi);

        /* Preuredjena lista:
           ispis broja elemenata i cijele liste. */

    printf("\n");
    printf(" Preuredjena lista (parni, neparni):\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);

        /* Obrisi cijelu listu. */

    prvi = obrisi_listu(prvi);

    return 0;
}
