#include <stdio.h>
#include <stdlib.h>

  /* Preuredjenje vezane liste - prvo parni, onda neparni.
     CUVA relativni poredak parnih, odnosno, neparnih elemenata.

     Algoritam 3:
     - Gradi jednu zasebnu sredjenu listu, koja je uvijek korektno spojena
       (nema spajanja na kraju).
     - Zasebna lista NIJE stalno uzemljena, uzemljava se na kraju.
     NIJE efikasan, zbog stalnog spajanja (azuriranje pokazivaca)!
  */

/* ======================================================================== */

    /* Tip za pokazivac na element liste. */

typedef struct _element *lista;

    /* Tip za element liste. */

typedef struct _element {
    int broj;            /* Sadrzaj je broj. */
    lista sljed;         /* Pokazivac na sljedeci element u listi. */
} element;

/* ======================================================================== */

    /* Preuredjuje listu tako da na pocetku liste budu svi parni,
       a zatim dolaze svi neparni elementi iz polazne liste.
       Relativni poredak parnih, odnosno, neparnih elemenata
       je ISTI kao u polaznoj listi. */

    /* Algoritam 3:
         Gradi jednu zasebnu sredjenu listu (parni i neparni dio su
       uvijek korektno spojeni), obradom jednog po jednog elementa.
       Sredjena lista NIJE stalno uzemljena (za povecanje efikasnosti),
       vec se uzemljava na kraju.

         Za CUVANJE relativnog poretka, nakon izbacivanja elementa iz
       polazne liste, radimo ubacivanje na KRAJ odgovarajuce podliste
       u sredjenoj listi. Zato pamtimo prvi i zadnji element u obje
       podliste (parni, neparni). Ako obje podliste nisu prazne, onda
       su uvijek korektno spojene (zadnji_parni->sljed = prvi_neparni).

         Unutar petlje za obradu, sljedbenik zadnjeg se NE postavlja
       na NULL. Uzemljenje se radi tek na samom kraju, iza petlje.
    */

lista preuredi_par_nepar_3(lista prvi)
{
        /* Inicijalizacije podliste parnih i podliste neparnih, u
           sredjenoj listi, na praznu listu.
           Za obje podliste pamtimo pokazivace na prvi i zadnji element.
           Pokazivace na prvi element inicijaliziramo na NULL.
           Pokazivace na zadnji element NE treba inicijalizirati na NULL. */
    lista prvi_parni = NULL, zadnji_parni;
    lista prvi_neparni = NULL, zadnji_neparni;

        /* Pokazivac na radni element (onaj kojeg obradjujemo). */
    lista radni;

        /* Petlja za prolaz po polaznoj listi.
           Napreduje za po jedan radni element. */

    while (prvi != NULL) {

            /* Izbaci prvi preostali element (= radni) i makni pocetak. */
        radni = prvi;
        prvi = prvi->sljed;

            /* Sad obradjujemo izbaceni radni element, tj. ubacujemo ga
               na KRAJ odgovarajuce podliste, ovisno o parnosti broja. */

        if (radni->broj % 2 == 0) {    /* Parni. */

                /* Ubaci ga na kraj podliste parnih. */
            if (prvi_parni == NULL)
                prvi_parni = radni;
            else    /* Ovdje zadnji_parni ne mora biti uzemljen. */
                zadnji_parni->sljed = radni;
            zadnji_parni = radni;    /* Ne mora biti uzemljen. */

                /* Spajanje parnog (prednjeg) i neparnog (straznjeg) dijela
                   sredjene liste mozemo napraviti na dva nacina.

                   1. nacin:
                   --------
                   Ako neparni dio nije prazan, spoji ga iza zadnji_parni.
                   Ako je neparni dio prazan, onda zadnji_parni ne mora biti
                   uzemljen (on je tada zadnji u cijeloj sredjenoj listi).
            if (prvi_neparni != NULL)
                zadnji_parni->sljed = prvi_neparni;

                   2. nacin:
                   --------
                   Uvijek spojimo prvi_neparni iza zadnji_parni, BEZ
                   provjere je li neparni dio neprazan. Ako je neparni dio
                   prazan, to uzemljuje parni = cijeli sredjeni dio.
                   Onda je zadnji_parni->sljed uvijek korektno postavljen.
                   To skracuje test kod uzemljenja cijele sredjene liste. */
            zadnji_parni->sljed = prvi_neparni;

                /* Za parni element, to spajanje se radi SVAKI puta kad
                   ubacimo element na kraj parnog dijela (kraj se mijenja).
                   Napomena: Ovo (stalno) spajanje parnog i neparnog
                   dijela USPORAVA algoritam, obzirom na alg. 1 i 2. */
        }
        else {    /* Neparni. */

                /* Ubaci ga na kraj podliste neparnih. */
            if (prvi_neparni == NULL) {

                    /* Neparni dio postaje neprazan, postavi prvi_neparni. */
                prvi_neparni = radni;

                    /* Ako parni dio nije prazan, onda spoji prvi_neparni
                       iza zadnji_parni. Za neparni element, to spajanje
                       se radi samo JEDNOM, kad podlista neparnih postane
                       neprazna (prvi_neparni se iza toga NE mijenja). */
                if (prvi_parni != NULL)
                    zadnji_parni->sljed = prvi_neparni;    /* = radni. */
            }
            else    /* Ovdje zadnji_neparni ne mora biti uzemljen. */
                zadnji_neparni->sljed = radni;
            zadnji_neparni = radni;    /* Ne mora biti uzemljen. */
        }
    }

        /* Na kraju, uzemljimo cijelu sredjenu listu, ako nije prazna.
           Ako koristimo 2. nacin spajanja (podlista parnih je korektno
           uzemljena ako je lista neparnih prazna), dovoljno je uzmeljiti
           samo podlistu neparnih, ako nije prazna (jer neparni idu na
           kraj sredjene liste). */
    if (prvi_neparni != NULL)
        zadnji_neparni->sljed = NULL;

        /* Za 1. nacin spajanja, to nije dovoljno. Ako nema neparnih,
           onda treba uzemljiti parnu podlistu (= cijela sredjena lista),
           ako nije prazna. Dakle, gornjem if-u treba dodati else dio:
    else
         if (prvi_parni != NULL)
             zadnji_parni->sljed = NULL;
        */

        /* Vrati pocetak sredjene liste. Ako ima parnih elemenata,
           to je prvi_parni. U protivnom (prvi_parni == NULL), to je
           prvi_neparni (cak i kad je on jednak NULL, tj. polazna lista
           je bila prazna). */

    if (prvi_parni == NULL)
        return prvi_neparni;
    else
        return prvi_parni;
}

/* ======================================================================== */

    /* Ispod su funkcije za osnovne operacije s listama. */

/* ======================================================================== */

lista kreiraj_straga(lista prvi, lista *p_zadnji, int broj)
{
        /* Vraca zadnji kroz varijabilni argument - pokazivac p_zadnji. */

    lista zadnji = *p_zadnji;
    lista novi = NULL;
/*
    if ((novi = (lista) malloc(sizeof(element))) == NULL)
*/
    novi = (lista) malloc(sizeof(element));
    if (novi == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(EXIT_FAILURE);    /* exit(1); */
    }

    novi->broj = broj;
    novi->sljed = NULL;

    if (prvi == NULL)
        prvi = novi;
    else
        zadnji->sljed = novi;

/* Ne treba:
    zadnji = novi; *p_zadnji = zadnji;
*/
    *p_zadnji = novi;    /* Vrati novi zadnji! */

    return prvi;
}

/* ======================================================================== */

lista obrisi_listu(lista prvi)
{
    lista pom;
/*
    while (prvi != NULL)
        prvi = obrisi_prvog(prvi);
*/
    while (prvi != NULL) {
        pom = prvi;
        prvi = prvi->sljed;
        free(pom);
    }

    return NULL;    /* <=> return prvi; */
}

/* ======================================================================== */

int broj_elemenata(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed)
        ++brojac;

    return brojac;
}

/* ======================================================================== */

void ispisi_listu(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed) {
        printf(" Element %2d, broj = %2d\n",
               ++brojac, pom->broj);
    }

    return;
}

/* ======================================================================== */

int main(void)
{
    lista prvi = NULL;    /* Prazna lista! */
    lista zadnji;
    int broj;

        /* Punjenje liste straga. */

        /* Citanje niza brojeva za listu, do broj = 0.
           To je oznaka kraj niza, ali nije clan niza. */
        /* Poredak brojeva u listi je isti kao redoslijed
           citanja, jer dodajemo na kraj liste. */

    printf("Upisite niz cijelih brojeva (0 = kraj).\n");
    do {
        printf("Sljedeci broj:\n");
        scanf("%d", &broj);
        if (broj != 0)
            prvi = kreiraj_straga(prvi, &zadnji, broj);
    } while (broj != 0);

        /* Ispis broja elemenata i cijele liste. */

    printf("\n");
    printf(" Polazna lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);


        /* Preuredjenje liste - prvo parni, onda neparni. */

    prvi = preuredi_par_nepar_3(prvi);

        /* Preuredjena lista:
           ispis broja elemenata i cijele liste. */

    printf("\n");
    printf(" Preuredjena lista (parni, neparni):\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);

        /* Obrisi cijelu listu. */

    prvi = obrisi_listu(prvi);

    return 0;
}
