#include <stdio.h>
#include <stdlib.h>

  /* Preuredjenje vezane liste - prvo parni, onda neparni.
     CUVA relativni poredak parnih, odnosno, neparnih elemenata.

     Algoritam 4:
     - Preuredjenje u samo jednoj (radnoj) listi (uvijek je sve spojeno),
       nema dodatne zasebne liste.
     - Preuredjeni dio se nalazi na pocetku radne liste, a iza toga
       je nesredjeni dio polazne liste.
     NIJE efikasan, zbog stalnog spajanja (azuriranje pokazivaca)!
  */

/* ======================================================================== */

    /* Tip za pokazivac na element liste. */

typedef struct _element *lista;

    /* Tip za element liste. */

typedef struct _element {
    int broj;            /* Sadrzaj je broj. */
    lista sljed;         /* Pokazivac na sljedeci element u listi. */
} element;

/* ======================================================================== */

    /* Preuredjuje listu tako da na pocetku liste budu svi parni,
       a zatim dolaze svi neparni elementi iz polazne liste.
       Relativni poredak parnih, odnosno, neparnih elemenata
       je ISTI kao u polaznoj listi. */

    /* Algoritam 4:
         Preuredjenje unutar jedne liste, obradom jednog po jednog
       elementa. Ako element vec NIJE na pravom mjestu u sredjenom
       komadu liste, onda ga izbacujemo s trenutnog mjesta i ubacujemo
       na propisno mjesto u vec sredjenom komadu.

         Za CUVANJE relativnog poretka, nakon izbacivanja odgovarajuceg
       elementa, radimo ubacivanje na KRAJ odgovarajuceg dijela u vec
       sredjenom komadu liste.

         To je puno lakse napraviti u slucaju kad se vec sredjeni komad
       liste nalazi na POCETKU liste (u PREDNJEM komadu radne liste),
       ISPRED trenutnog "radnog" mjesta, a nastavak liste (od tog mjesta
       nadalje) jos nije sredjen.

         Zbog poretka "prvo parni, onda neparni" u sredjenom komadu,
       obrada trenutnog radnog elementa je:
         - neparni element se preskace (ostavlja gdje je), jer se vec
           nalazi na KRAJU neparnog dijela sredjenog komada liste
           (do ovog mjesta),
         - parni element se izbacuje s trenutnog mjesta i ubacuje na
           KRAJ parnog dijela sredjenog komada liste. Ovo izbacivanje
           i ubacivanje parnog se radi samo kad treba, tj. kad taj
           element vec NIJE na svom pravom mjestu (<=> ima neparnih
           u sredjenom dijelu).
       Zato pamtimo prvi i zadnji element u parnom (= prednjem) dijelu
       sredjenog komada liste, a za neparni (= straznji) dio sredjenog
       komada, dovoljno je pamtiti samo prvi element. Zadnji sredjeni
       element je upravo "prethodnik" elementa kojeg obradjujemo,
       tj. zadnji_neparni <=> preth (ako ima neparnih).
    */

lista preuredi_par_nepar_4(lista prvi)
{
        /* Inicijalizacije podliste parnih (prvi i zadnji) i podliste
           neparnih (samo prvi), u sredjenom (pocetnom) komadu liste,
           na praznu listu.
           Pokazivace na prvi element inicijaliziramo na NULL.
           Pokazivac na zadnji parni NE treba inicijalizirati na NULL. */
    lista prvi_parni = NULL, zadnji_parni;
    lista prvi_neparni = NULL;

        /* Pokazivac na radni element (onaj kojeg obradjujemo).
           Inicijaliziramo ga na prvi (pocetno radno mjesto u listi). */
    lista radni = prvi;

        /* Pokazivac na element ispred onog kojeg obradjujemo.
           NE treba ga inicijalizirati na NULL.
           Dodatno, preth pokazuje na zadnji element u sredjenom komadu
           liste, cim taj komad NIJE prazan. Ako podlista neparnih nije
           prazna, onda je zadnji_neparni = preth. Ako nema neparnih,
           a podlista parnih nije prazna, onda je zadnji_parni = preth. */
    lista preth;

        /* Petlja za prolaz po polaznoj listi.
           Napreduje za po jedan radni element. */

    while (radni != NULL) {

            /* Pokazivac radni pokazuje na element kojeg obradjujemo,
               ovisno o parnosti broja (ostavi ili izbaci/ubaci). */

        if (radni->broj % 2 == 0) {    /* Parni. */

                /* Parni element se izbacuje s trenutnog mjesta i
                   ubacuje na kraj parnog dijela vec sredjenog (prednjeg)
                   komada liste. Ovo izbacivanje i ubacivanje parnog
                   se radi samo kad treba, tj. kad taj element vec NIJE
                   na svom pravom mjestu u sredjenom komadu. Uociti:
                   on NIJE na dobrom mjestu <=> u sredjenom dijelu ima
                   bar jedan neparni (izmedju sredjenih parnih i ovog)! */

            if (prvi_neparni == NULL) {    /* Nema izbaci, ubaci. */

                    /* Nema neparnih u ranije sredjenom dijelu, pa nema
                       izbaci, ubaci - parni element ostaje na ovom mjestu.
                       Azuriraj pokazivace za parnu podlistu. Ako je to
                       prvi parni, onda postavi prvi_parni na njega. */
                if (prvi_parni == NULL)
                    prvi_parni = radni;

                    /* Ovo je novi zadnji_parni (cak i ako je prvi). */
                zadnji_parni = radni;

                    /* Uociti da NEMA promjene sljedbenika zadnjeg parnog,
                       jer (jos) nema neparnih. Iza zadnji_parni je
                       (nesredjeni) nastavak liste - tj. lista ostaje vezana.
                       Ako uopce nema neparnih u listi, onda prethodna
                       naredba korektno "uzemljuje" listu, jer zadnji radni
                       tada pokazuje na zadnji element u listi (njegov
                       sljedbenik je NULL). */

                    /* Makni preth na ovog za sljedeci korak ("dodali"
                       smo ovog na sam KRAJ sredjenog dijela).
                       Pomak radnog nije ovdje, nego na dnu petlje,
                       jer je zajednicki za sve grane algoritma. */
                preth = radni;
            }
            else {    /* prvi_neparni != NULL. */

                    /* Ima bar jedan neparni u ranije sredjenom dijelu.
                       IZBACI ovog parnog s trenutnog mjesta = promijeni
                       sljedbenika od preth (preth je sigurno korektno
                       definiran, jer ovaj radni NE moze biti prvi u listi).
                       Uociti da se preth NE mijenja za sljedeci korak
                       zato sto se radni ubacuje negdje ISPRED kraja
                       sredjenog dijela liste (ima neparnih izmedju).
                       Ovo spaja sredjeni i nesredjeni dio liste, a novi
                       radni je zapamcen u preth->sljed! */
                preth->sljed = radni->sljed;

                    /* UBACI radni element na kraj parnih u sredjenom
                       dijelu liste. Azuriraj pokazivace za parnu podlistu.
                       Ako je to prvi parni, onda postavi prvi_parni na
                       njega. U protivnom, stavi ga kao sljedbenika od
                       zadnji_parni. */
                if (prvi_parni == NULL)
                    prvi_parni = radni;
                else
                    zadnji_parni->sljed = radni;

                    /* Ovo je novi zadnji_parni (cak i ako je prvi). */
                zadnji_parni = radni;

                    /* Spoji neparni sredjeni dio (nije prazan) iza
                       (novog) zadnjeg u parnom dijelu, tj. sljedbenik
                       zadnjeg parnog je prvi neparni (ima ga)!
                       Napomena: Ovo (stalno) spajanje parnog i neparnog
                       dijela USPORAVA algoritam, obzirom na alg. 1 i 2. */
                zadnji_parni->sljed = prvi_neparni;

                    /* Uociti da to MIJENJA radni->sljed i zato novi
                       radni vise NIJE radni->sljed, nego preth->sljed
                       (= ranije zapamcen stari/pocetni radni->sljed).
                       Pomak radnog nije ovdje, nego na dnu petlje,
                       jer je zajednicki za sve grane algoritma. */
            }
        }    /* Kraj obrade parnog. */

        else {    /* Neparni. */

                /* Neparni element se preskace, tj. ostavlja gdje je,
                   jer se vec nalazi na KRAJU sredjenog dijela neparnih.
                   Azuriraj pokazivac (samo prvi) za neparnu podlistu. Ako
                   je to prvi neparni, postavi prvi_neparni na njega. */

            if (prvi_neparni == NULL)
                prvi_neparni = radni;

                /* Makni preth na ovog za sljedeci korak (preskakanjem
                   smo "dodali" ovog na sam KRAJ sredjenog dijela).
                   Pomak radnog nije ovdje, nego na dnu petlje,
                   jer je zajednicki za sve grane algoritma. */
            preth = radni;
        }    /* Kraj obrade neparnog. */

            /* Makni radni na sljedeci element u listi (prvi
               u nesredjenom dijelu = sljedbenik od preth). */
        radni = preth->sljed;

/* Napomena: Ako pomak radnog na "sljedeci" element napravimo ovako:
        radni = radni->sljed;
   algoritam (zacudo) i dalje radi KOREKTNO, ali se USPORAVA.
     Objasnjenje: Problem nastaje samo kad NE preskacemo radni element
   nego ga zaista ubacujemo negdje ISPRED kraja sredjenog dijela,
   tj. kad je preth->sljed != radni->sljed.
   To se dogadja samo kad je radni element paran, a u listi IMA neparnih.
   Nakon ubacivanja na kraj parnih i spajanja (nepraznog) neparnog dijela,
   vrijedi:
       zadnji_parni = radni  i  zadnji_parni->sljed = prvi_neparni,
   pa je radni->sljed = prvi_neparni. Dakle, novi radni onda postaje
   prvi_neparni (a ne prvi nesredjeni). Stoga algoritam "preskace"
   SVE sljedece neparne (oni sredjeni i eventualni pocetni nesredjeni),
   tj. nepotrebno PONOVNO "obradjuje" sve sredjene neparne. To se
   ponavlja za SVAKI sljedeci parni element u nesredjenom dijelu!
   Zbog (visestrukog) preskakanja sredjenih neparnih, algoritam i dalje
   radi korektno, samo je bitno SPORIJI.
*/
    }

        /* Vrati pocetak preuredjene liste. Ako ima parnih elemenata,
           to je prvi_parni. U protivnom (prvi_parni == NULL), to je
           prvi_neparni (cak i kad je on jednak NULL, tj. polazna lista
           je bila prazna). */

    if (prvi_parni == NULL)
        return prvi_neparni;
    else
        return prvi_parni;
}

/* ======================================================================== */

    /* Ispod su funkcije za osnovne operacije s listama. */

/* ======================================================================== */

lista kreiraj_straga(lista prvi, lista *p_zadnji, int broj)
{
        /* Vraca zadnji kroz varijabilni argument - pokazivac p_zadnji. */

    lista zadnji = *p_zadnji;
    lista novi = NULL;
/*
    if ((novi = (lista) malloc(sizeof(element))) == NULL)
*/
    novi = (lista) malloc(sizeof(element));
    if (novi == NULL) {
        printf("Alokacija nije uspjela.\n");
        exit(EXIT_FAILURE);    /* exit(1); */
    }

    novi->broj = broj;
    novi->sljed = NULL;

    if (prvi == NULL)
        prvi = novi;
    else
        zadnji->sljed = novi;

/* Ne treba:
    zadnji = novi; *p_zadnji = zadnji;
*/
    *p_zadnji = novi;    /* Vrati novi zadnji! */

    return prvi;
}

/* ======================================================================== */

lista obrisi_listu(lista prvi)
{
    lista pom;
/*
    while (prvi != NULL)
        prvi = obrisi_prvog(prvi);
*/
    while (prvi != NULL) {
        pom = prvi;
        prvi = prvi->sljed;
        free(pom);
    }

    return NULL;    /* <=> return prvi; */
}

/* ======================================================================== */

int broj_elemenata(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed)
        ++brojac;

    return brojac;
}

/* ======================================================================== */

void ispisi_listu(lista prvi)
{
    lista pom;
    int brojac = 0;

    for (pom = prvi; pom != NULL; pom = pom->sljed) {
        printf(" Element %2d, broj = %2d\n",
               ++brojac, pom->broj);
    }

    return;
}

/* ======================================================================== */

int main(void)
{
    lista prvi = NULL;    /* Prazna lista! */
    lista zadnji;
    int broj;

        /* Punjenje liste straga. */

        /* Citanje niza brojeva za listu, do broj = 0.
           To je oznaka kraj niza, ali nije clan niza. */
        /* Poredak brojeva u listi je isti kao redoslijed
           citanja, jer dodajemo na kraj liste. */

    printf("Upisite niz cijelih brojeva (0 = kraj).\n");
    do {
        printf("Sljedeci broj:\n");
        scanf("%d", &broj);
        if (broj != 0)
            prvi = kreiraj_straga(prvi, &zadnji, broj);
    } while (broj != 0);

        /* Ispis broja elemenata i cijele liste. */

    printf("\n");
    printf(" Polazna lista:\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);


        /* Preuredjenje liste - prvo parni, onda neparni. */

    prvi = preuredi_par_nepar_4(prvi);

        /* Preuredjena lista:
           ispis broja elemenata i cijele liste. */

    printf("\n");
    printf(" Preuredjena lista (parni, neparni):\n");
    printf(" Broj elemenata u listi = %d\n",
           broj_elemenata(prvi));
    ispisi_listu(prvi);

        /* Obrisi cijelu listu. */

    prvi = obrisi_listu(prvi);

    return 0;
}
