/* Zadatak 1B:

*/

#include <stdio.h>
#include <stdlib.h>

    /* Globalna deklaracija tipa. */

typedef struct {
    char prezime[25];
    char ime[25];
    int broj;
    char drzava[3];
    double rezultat[16];
} biatlonac;

    /* Pomocna funkcija.
       Preskace niz od nula ili vise praznina u datoteci ulaz
       i vraca prvi znak koji nije praznina (blank). */

int preskoci_praznine(FILE* ulaz)
{
    int ch;
    while ((ch = fgetc(ulaz)) == ' ');
    return ch;
}

    /* Funkcija za citanje jednog biatlonca. */

biatlonac* ucitaj(FILE* ulaz)
{
    biatlonac* p_bia;
    int sep = '/';
    int ch, m, i;
    int mm, ss, desetinke;

    p_bia = (biatlonac*) malloc(sizeof(biatlonac));

        /* 1. red. */
    fscanf(ulaz, "%[^,],%*c%[^\n]%*c", p_bia->prezime, p_bia->ime);

        /* 2. red. */
/* Kraticu mozemo ucitati petljom (znak po znak), sljedecim
   nizom naredbi:

    for (i = 0; i < 3; ++i)
        fscanf(ulaz, "%c", &(p_bia->drzava[i]));

   Umjesto toga, cijelu kraticu citamo odjednom - kao polje s 3 znaka:
*/
    fscanf(ulaz, "%3c", p_bia->drzava);
    fgetc(ulaz);
    fscanf(ulaz, "%d\n", &(p_bia->broj));

        /* 3. red. */
    m = 0;
        /* Petlja, sve dok ne ucitamo '\n' (kraj reda). */
    do {
        ch = preskoci_praznine(ulaz);

            /* Ako je znak ch == sep ili '\n' (kraj reda),
               pripadni blok je prazan (nema vremena). */
        if (ch == sep || ch == '\n') {
            p_bia->rezultat[m] = -1.0;
        }
            /* U protivnom, blok nije prazan (ima vrijeme),
               znak ch mora biti znamenka i to su desetice minuta! */
        else {
/* 1. mogucnost:
   Znak ch treba pretvoriti u znamenku desetica (znak -> broj).
   Ucitamo nastavak vremena (bez prvog znaka), tj. znamenku jedinica
   za minute i ostatak vremena u zadanom formatu.
   Onda sve pretvorimo u sekunde, s tim da znamenku desetica za minute
   treba korektno dodati (pomnoziti s 10 i dodati jedinicama za minute).
   To se moze napraviti sljedecim naredbama:

            fscanf(ulaz, "%d:%d,%d", &mm, &ss, &desetinke);
            mm = (ch - '0') * 10 + mm;
            p_bia->rezultat[m] = mm * 60.0 + ss + desetinke / 10.0;

   2. mogucnost:
   Vratimo znak ch NATRAG u spremnik za ponovno citanje (ungetc),
   ucitamo CIJELO vrijeme u zadanom formatu i sve pretvorimo u sekunde:
*/
            ungetc(ch, ulaz);
            fscanf(ulaz, "%d:%d,%d", &mm, &ss, &desetinke);
            p_bia->rezultat[m] = mm * 60.0 + ss + desetinke / 10.0;

            ch = preskoci_praznine(ulaz);
        }
            /* Povecaj broj obradjenih blokova. */
        ++m;
            /* Ovdje je sigurno ch == sep ili '\n'.
               Provjeri kraj reda = prekid petlje,
               tj. nastavi petlju ako ch nije '\n'. */
    } while (ch != '\n');

        /* Postavi preostala vremena, za koja nije bilo blokova. */
    for (i = m; i < 16; ++i)
        p_bia->rezultat[i] = -1.0;

        /* Vrati pokazivac na strukturu. */
    return p_bia;
}

    /* Glavni program za demo. */

int main(void)
{
    FILE *ulaz;
    char* ime = "1b.txt";
    biatlonac* p_bia;
    int k, i;
    int ch;

        /* Otvaranje ulazne tekst-datoteke. */
    if ((ulaz = fopen(ime, "r")) == NULL) {
        fprintf(stderr, "Ne mogu citati: %s!\n", ime);
        exit(1);
    }

        /* Ucitaj sve biatlonce, do kraja datoteke. */
    k = 0;
    while (1) {
        ch = fgetc(ulaz);
        if (ch == EOF) break;

            /* Ako nije EOF, onda smo upravo ucitali prvi znak
               u sljedecem redu (pocetak sljedeceg podatka).
               Umjesto posebne obrade tog znaka, vratimo ga
               NATRAG u spremnik, za ponovno citanje.
               Funkcija ucitaj ce ga jos jednom procitati. */
        ungetc(ch, ulaz);

        p_bia = ucitaj(ulaz);

            /* Ispis pripadnih podataka za kontrolu. */
        printf(" %d. biatlonac:\n", k + 1);
        printf("   Prezime = \"%s\"\n", p_bia->prezime);
        printf("       Ime = \"%s\"\n", p_bia->ime);
        printf("      Broj = %d\n", p_bia->broj);
/* Kraticu mozemo napisati petljom (znak po znak), sljedecim
   nizom naredbi:

        printf("    Drzava = ");
        for (i = 0; i < 3; ++i)
            printf("%c", p_bia->drzava[i]);
        printf("\n");

   Umjesto toga, cijelu kraticu pisemo odjednom - kao string,
   ali tako da pisemo najvise 3 znaka iz polja:
*/
        printf("    Drzava = %.3s\n", p_bia->drzava);
            /* Rezultati. */
        for (i = 0; i < 16; ++i)
            printf("   Rezultat[%2d] = %6.1f\n", i, p_bia->rezultat[i]);
        printf("\n");

        ++k;
        free(p_bia);
    }

    fclose(ulaz);

    return 0;
}
