#include <stdio.h>

  /* Primjer: Citanje stringa u jednom redu standardnog ulaza.
     Funkcija my_gets:
       Zamjena za funkciju gets (C90 standard, izbacena u C11).
     Slicno funkciji gets_s iz C11 standarda, ali s manje ogranicenja,
     tj. my_gets je fleksibilnija implementacija te funkcije.

     Ovdje je samo glavni program za testiranje citanja.
     Implementacija funkcije my_gets je u posebnoj datoteci my_gets.c.

     Koristimo zasebno prevodjenje i linkanje - umjesto #include,
     pisemo extern i prototip funkcije my_gets.
  */

extern char *my_gets(char *s, int n);


    /* Glavni program. Za testiranje, ulaz je preusmjeren
       na odgovarajucu datoteku, koja sadrzi izabrane primjere.
         Moze se koristiti i bez toga - interaktivno cita
       string po string (svaki string = jedan red), sve dok
       se ne upise CTRL-Z = kraj datoteke.
         Za test, prva dva poziva funkcije my_gets namjerno imaju
       pogresne argumente. Zato se stringovi u prva dva reda ulaza
       preskacu (red se procita, ali nema spremanja u string). */

int main(void)
{
        /* Najveci broj znakova koji stane u polje za citanje. */
    int max_znakova = 20;
        /* U polje s stane najvise 20 znakova.
           Od toga, najvise 19 znakova je sadrzaj stringa,
           tako da IZA sadrzaja ima jos jedno mjesto za
           nul-znak (\0) = kraj stringa. */
    char s[20];
        /* NULL pokazivac na znak, za test argumenata kod poziva. */
    char *s_NULL = NULL;
        /* Izlazna vrijednost funkcije my_gets. */
    char *izlaz;

        /* Logicka vrijednost za nastavak citanja. */
    int nastavi = 1;
        /* Redni broj reda (linije) na ulazu. */
    int br_reda = 0;

        /* Prva dva reda (linije) su testovi s pogresnim argumentima
           u pozivu funkcije my_gets. */
    ++br_reda;
    s[0] = '?';    /* Za test promjene u polju s (nije string). */
    printf("Test argumenata - NULL pointer (red se preskace)!\n");
    printf("Upisite %d. red (string):\n", br_reda);
        /* Pogresan poziv - prvi argument je NULL pokazivac. */
    izlaz = my_gets(s_NULL, max_znakova);
    if (izlaz == NULL && s[0] == '?')
        printf("  [OK] Pogresan argument, string nepromijenjen.\n");
    else
        printf("  GRESKA u testu argumenata!\n");
    if (feof(stdin) || ferror(stdin)) {
        printf("  Citanje reda: Kraj datoteke ili greska.\n");
        return 0;
    }

    ++br_reda;
    s[0] = '?';    /* Za test promjene u polju s (nije string). */
    printf("\n");
    printf("Test argumenata - n < 0 (red se preskace)!\n");
    printf("Upisite %d. red (string):\n", br_reda);
        /* Pogresan poziv - drugi argument nije pozitivan. */
    izlaz = my_gets(s, -1);
    if (izlaz == NULL && s[0] == '?')
        printf("  [OK] Pogresan argument, string nepromijenjen.\n");
    else
        printf("  GRESKA u testu argumenata!\n");
    if (feof(stdin) || ferror(stdin)) {
        printf("  Citanje reda: Kraj datoteke ili greska.\n");
        return 0;
    }

        /* Petlja za citanje "red-po-red", s korektnim argumentima,
           sve dok varijabla nastavi ima vrijednost istina (nije 0).
           Citanje se prekida, tj. nastavi posaje laz (0), ako i samo
           ako funkcija my_gets vrati NULL pokazivac.
             Uz korektne argumente za my_hets, to se moze dogoditi
           samo kad dodje do kraja datoteke ili greske. */
    do {
            /* Povecaj redni broj reda (linije), odnosno, stringa. */
        ++br_reda;
        printf("\n");
        printf("Upisite %d. red (string):\n", br_reda);

            /* Citanje reda/stringa. */
        izlaz = my_gets(s, max_znakova);

            /* Korektan poziv => my_gets MORA vratiti korektan string.
               Ispisi ga odmah, PRIJE provjere izlazne vrijednosti. */
        printf("  Ucitani string: \"%s\"\n", s);

            /* Provjera izlazne vrijednosti NULL. */
        if (izlaz == NULL) {
                /* Ocekivani razlog = kraj datoteke. */
            if (feof(stdin)) {
                printf("  Kraj datoteke prilikom citanja.\n");
                    /* Provjera vracenog stringa - mora biti prazan. */
                if (s[0] != '\0')
                    printf("  GRESKA: vraceni string nije prazan!\n");
            }
                /* Neocekivani, ali moguci razlog = greska kod citanja. */
            if (ferror(stdin)) {
                printf("  Greska na ulazu prilikom citanja.\n");
            }

                /* Ovo ispod se NE bi smjelo dogoditi.
                   Za test, provjeravamo i tu mogucnost. */
            if (!feof(stdin) && !ferror(stdin)) {
                printf("  GRESKA: izlaz NULL, a nije feof ili ferror!\n");
            }

                /* Prekini citanje. */
            nastavi = 0;
        }

    } while (nastavi);

    return 0;
}
