#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

  /* Program za sortiranje datoteke cijelih brojeva.
     Algoritam = Insertion_sort:
       Sortiranje (sortiranim) umetanjem - sortirani dio je spijeda,
       uz sekvencijalno trazenje prema pocetku.
       Izbjegava nepotrebna pisanja ako je novi element najveci.
       Za datoteke, dva kopiranja su nuzna - kod citanja.
     Predlozak: ins_1 (Prog_1, 13_PROGS). */

  /* Ubrzana varijanta:
     - koristi sekvencijalno citanje unaprijed,
     - koristi lokalnost pomaka elemenata, za pisanje kod pomaka.
  */

/* ======================================================================== */

    /* Insertion_sort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Ubrzana varijanta. */

void f_insertion_sort(FILE *f_obj,
                   long int n_obj, size_t obj_size,
                   int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j;
    long int pos_i;
    obj_type x_i, x_jm1;    /* x_i = temp = x[i], x_jm1 = x[j - 1]. */

        /* Pomak za tri objekta natrag kod citanja unatrag,
           nakon pomaka (kopiranja) objekta. */
    long int tri_natrag = -3L * obj_size;

        /* Logicka varijabla koja pamti je li bilo pomaka u
           proslom koraku (prethodnom prolazu kroz petlju po i).
           Za prvi prolaz, inicijaliziramo ju na 1 (kao da je
           bilo pomaka), jer treba ucitati oba elementa. */
    int bilo_pomaka = 1;


        /***** Insertion_sort. *****/

        /* Pozicija na kojoj se nalazi sljedeci element x[i] za
           ubacivanje. Inicijalizacija odgovara prvom elementu x[0]. */
    pos_i = 0L ;

        /* Vanjska petlja za prolaze kroz datoteku.
           Indeks i je indeks sljedeceg elementa x[i],
           kojeg sortirano ubacujemo u sortirani prednji dio. */
    for (i = 1L; i < n_obj; ++i) {

            /* Pomakni poziciju elementa x[i] za sljedeci prolaz. */
        pos_i += obj_size;

            /* Uociti: Ako u prethodnom prolazu NIJE bilo pomaka,
               onda je prethodni x[i] upravo novi x[i - 1].
               Dakle, dovoljno je ucitati samo novi x[i].
               U protivnom, ako je bilo pomaka (ili je ovo prvi prolaz),
               onda moramo ucitati oba elementa x[i - 1] i x[i]. */

        if (bilo_pomaka) {
                /* Postavi poziciju za x[i - 1]. */
            postavi_fpos(f_obj, pos_i - obj_size, SEEK_SET);

                /* Ucitaj element x[i - 1] u x_jm1.
                   Ovo kopiranje je nuzno kod datoteka. */
            ucitaj_obj(f_obj, &x_jm1, obj_size);
        }
        else {
                /* Postavi poziciju za x[i] (preskoci x[i - 1]). */
            postavi_fpos(f_obj, pos_i, SEEK_SET);

                /* Postavi x_jm1 (novi element x[i - 1])
                   kao prethodnu vrijednost x_i = x[i]. */
            x_jm1 = x_i;
        }

            /* Ucitaj element x[i] u x_i = temp.
               Ovo kopiranje je nuzno kod datoteka. */
        ucitaj_obj(f_obj, &x_i, obj_size);

            /* Usporedba x_i = x[i] i x_jm1 = x[i - 1]. */
        if (obj_cmp(&x_i, &x_jm1) < 0) {

                /* Ima pomaka. Ovdje je x_i = temp = x[i].
                   Pozicija se opet postavlja ISPRED x[i], kao da smo
                   prvo ucitali x[i], a ZATIM x[i - 1]. To je nuzno
                   za korektno pisanje na pocetku pomaka. */
            postavi_fpos(f_obj, pos_i, SEEK_SET);

                /* Silazna petlja po j, za pomake (kopiranja). */
/*****
            j = i;
            do {
                x[j] = x[j - 1]; --j;
            } while (j >= 1 && x[j - 1] > temp);
            x[j] = temp;
*****/
            j = i;
            do {
                    /* Kopiraj x_jm1 = x[j - 1] u x[j] (pomak).
                       Pisanje se sekvencijalno nastavlja na istom mjestu
                       gdje je prethodno citanje stalo. Pozicija je vec
                       korektno postavljena (ispred petlje ili na dnu). */
                napisi_obj(f_obj, &x_jm1, obj_size);

                    /* Smanji j, provjeri je li j >= 1 za nastavak. */
                --j;
                if (j < 1L) {

                        /* Element x[i] je novi najmanji (ispred svih).
                           Prije smanjenja j na 0, napisali smo x[0]
                           na mjesto x[1], a sad se treba postaviti za
                           umetanje x_i na mjesto x[0], tj. na pocetak
                           datoteke - ispred x[0]. Ekvivalentni pomak
                           je za dva objekta unatrag (ne moze za tri). */
                    postavi_fpos(f_obj, 0L, SEEK_SET);
                    break;
                }

                    /* Pomak unatrag za tri mjesta, za sljedeci x[j - 1]. */
                postavi_fpos(f_obj, tri_natrag, SEEK_CUR);

                    /* Ucitaj element x[j - 1]. */
                ucitaj_obj(f_obj, &x_jm1, obj_size);

                    /* Sljedece pisanje se sekvencijalno nastavlja na istom
                       mjestu gdje je prethodno citanje stalo. Zbog promjene
                       s citanja na pisanje, postavljamo istu poziciju. */
                postavi_fpos(f_obj, 0L, SEEK_CUR);

                    /* Usporedba x_jm1 = x[j - 1] i x_i = temp
                       je ugradjena u uvjet do-while petlje. */
            } while (obj_cmp(&x_jm1, &x_i) > 0);

                /* Umetanje: napisi x_i = temp na mjesto x[j].
                   Ne treba postaviti poziciju, vec je postavljena. */
            napisi_obj(f_obj, &x_i, obj_size);

                /* Stavi bilo_pomaka na 1, za sljedeci korak. */
            bilo_pomaka = 1;
        }
        else {
                /* Nema pomaka. Stavi bilo_pomaka na 0, za sljedeci korak. */
            bilo_pomaka = 0;
        }
    }

        /* Postavi se na pocetak datoteke (za svaki slucaj). */
    rewind(f_obj);

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    f_insertion_sort(f_obj, n_obj, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
