#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

  /* Program za sortiranje datoteke cijelih brojeva.
     Algoritam = Insertion_sort:
       Sortiranje (sortiranim) umetanjem - sortirani dio je spijeda,
       uz sekvencijalno trazenje prema pocetku.
       Izbjegava nepotrebna pisanja ako je novi element najveci.
       Za datoteke, dva kopiranja su nuzna - kod citanja.
     Predlozak: ins_1a  (Prog_1, 13_PROGS),
                ins_1af (ovdje, na S_Polje). */

  /* Ubrzana varijanta:
     - koristi sekvencijalno citanje unaprijed,
     - koristi lokalnost pomaka elemenata, za pisanje kod pomaka.
  */

/* ======================================================================== */

    /* Insertion_sort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Ubrzana varijanta. */

void f_insertion_sort(FILE *f_obj,
                   long int n_obj, size_t obj_size,
                   int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j;
    long int pos_i;
    obj_type x_i, x_jm1;    /* x_i = temp = x[i], x_jm1 = x[j - 1]. */

        /* Pomak za dva objekta natrag kod citanja unatrag,
           prije pocetka pomaka. */
    long int dva_natrag = -2L * obj_size;

        /* Pomak za tri objekta natrag kod citanja unatrag,
           nakon pomaka (kopiranja) objekta. */
    long int tri_natrag = -3L * obj_size;


        /***** Insertion_sort. *****/

        /* Pozicija od koje pocinje citanje u petlji po i.
           Na njoj se nalazi sljedeci element x[i] za ubacivanje.
           Inicijalizacija odgovara prvom elementu, za i = 0. */
    pos_i = 0L ;

        /* Vanjska petlja za prolaze kroz datoteku.
           Indeks i je indeks sljedeceg elementa x[i],
           kojeg sortirano ubacujemo u sortirani prednji dio. */
    for (i = 1L; i < n_obj; ++i) {

            /* Pomakni poziciju elementa x[i] za sljedeci prolaz. */
        pos_i += obj_size;

            /* Postavi poziciju za sljedeci x[i]. */
        postavi_fpos(f_obj, pos_i, SEEK_SET);

            /* Ucitaj element x[i] u x_i = temp.
               Ovo kopiranje je nuzno kod datoteka. */
        ucitaj_obj(f_obj, &x_i, obj_size);

            /* Silazna petlja po j, za preskakanje
               x[j - 1] > x_i = temp i pomake (kopiranja). */
/*****
        for (j = i; j >= 1 && x[j - 1] > temp; --j)
            x[j] = x[j - 1];
        if (j < i)
            x[j] = temp;
*****/
            /* Pomak unatrag za dva mjesta, zbog sljedece for petlje,
               koja sigurno ima bar jedan prolaz. Ucitali smo x[i],
               tj. pozicija je narasla IZA njega. U petlji moramo ucitati
               x[i - 1] (za j = i), a treba se postaviti ISPRED njega. */
        postavi_fpos(f_obj, dva_natrag, SEEK_CUR);

        for (j = i; j >= 1L; --j) {

                /* Ucitaj element x[j - 1]. */
            ucitaj_obj(f_obj, &x_jm1, obj_size);

                /* Pisanje se sekvencijalno nastavlja na istom mjestu
                   gdje je prethodno citanje stalo. Zbog promjene s
                   citanja na pisanje, postavljamo istu poziciju.
                   To vrijedi za pisanje kod pomaka i kod umetanja. */
            postavi_fpos(f_obj, 0L, SEEK_CUR);

                /* Usporedba x_jm1 = x[j - 1] i x_i = temp.
                   Ako je x[j - 1] <= temp, prekini petlju. */
            if (obj_cmp(&x_jm1, &x_i) <= 0) break;

                /* U protivnom, kopiraj x_jm1 = x[j - 1] u x[j] (pomak). */
            napisi_obj(f_obj, &x_jm1, obj_size);

                /* Test prekida petlje za j == 1L (PRIJE smanjenja j),
                   da izbjegnemo pokusaj pomaka ispred pocetka datoteke. */
            if (j == 1L) {

                    /* Element x[i] je novi najmanji (ispred svih).
                       Napisali smo x[0] na mjesto x[1], a sad se treba
                       postaviti za umetanje x_i na mjesto x[0], tj. na
                       pocetak datoteke - ispred x[0]. Ekvivalentni pomak
                       je za dva objekta unatrag (ne moze za tri). */
                postavi_fpos(f_obj, 0L, SEEK_SET);

                    /* Smanji j na 0, jer izlazimo prije dna petlje. */
                j = 0L;
                break;
            }

                /* Pomak unatrag za tri mjesta, za sljedeci x[j - 1]. */
            postavi_fpos(f_obj, tri_natrag, SEEK_CUR);
        }

            /* Umetanje: napisi x_i = temp na mjesto x[j].
               Izbjegava pisanje na isto mjesto, ako je j == i.
               Ne treba postaviti poziciju, vec je postavljena. */
        if (j < i)
            napisi_obj(f_obj, &x_i, obj_size);
    }

        /* Postavi se na pocetak datoteke (za svaki slucaj). */
    rewind(f_obj);

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    f_insertion_sort(f_obj, n_obj, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
