#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

  /* Program za sortiranje datoteke cijelih brojeva.
     Algoritam = QuickSort:
       Kljucni element je prvi u danom dijelu polja [l, d]
       i njega dovodimo na pravo mjesto.
     Predlozak: qsort_1a (Prog_2, 01_PROGS). */
  /* Ideja: Filip Barl, 2. kolokvij iz Prog2, 2010.g. */

  /* Ubrzana varijanta:
     - koristi sekvencijalno citanje unaprijed i pomake unatrag,
     - pamti pozicije i vrijednosti kod zamjena objekata.
  */

/* ======================================================================== */

    /* QuickSort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Ubrzana varijanta. */

void f_quick_sort(FILE *f_obj,
                  long int l, long int d, size_t obj_size,
                  int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j;
    long int pos_l, pos_i, pos_j;
    obj_type x_l, x_i, x_j;    /* x_l = x[l], x_i = x[i], x_j = x[j]. */

        /* Pomak za dva objekta natrag kod citanja unatrag. */
    long int dva_natrag = -2L * obj_size;


        /***** Quick_sort. *****/

    if (l < d) {

            /* Postavi poziciju prvog elementa x[l]. */
        pos_l = l * obj_size;
        postavi_fpos(f_obj, pos_l, SEEK_SET);

            /* Ucitaj element x[l] u x_l. */
        ucitaj_obj(f_obj, &x_l, obj_size);

        i = l + 1L;
        j = d;

            /* Pozicije elemenata x[i] = x[l + 1] i x[j] = x[d]. */
        pos_i = pos_l + obj_size;    /* = ftell(f_obj); */
        pos_j = d * obj_size;

            /* Prolaz mora i za i == j */
        while (i <= j) {

                /* Postavi poziciju za sljedeci x[i]. */
            postavi_fpos(f_obj, pos_i, SEEK_SET);

                /* Pomak i unaprijed, preskakanjem x[i] <= x[l]. */
            while (i <= d) {

                    /* Ucitaj element x[i] u x_i. */
                ucitaj_obj(f_obj, &x_i, obj_size);

                if (obj_cmp(&x_i, &x_l) > 0) break;
                ++i;
            }
                /* Zapamti poziciju zadnjeg x[i]. */
            pos_i = ftell(f_obj) - obj_size;


                /* Postavi poziciju za sljedeci x[j]. */
            postavi_fpos(f_obj, pos_j, SEEK_SET);

                /* Pomak j unatrag, preskakanjem x[j] > x[l]. */
            while (1) {

                    /* Ucitaj element x[j] u x_j. */
                ucitaj_obj(f_obj, &x_j, obj_size);

                if (obj_cmp(&x_j, &x_l) <= 0) break;
                --j;

                    /* Pomak unatrag za dva mjesta, za sljedeci x[j]. */
                postavi_fpos(f_obj, dva_natrag, SEEK_CUR);
            }
                /* Zapamti poziciju zadnjeg x[j]. */
            pos_j = ftell(f_obj) - obj_size;

            if (i < j) {
                    /* Zamjena x[i] i x[j], na pozicijama pos_i, pos_j. */
                postavi_fpos(f_obj, pos_j, SEEK_SET);
                napisi_obj(f_obj, &x_i, obj_size);
                postavi_fpos(f_obj, pos_i, SEEK_SET);
                napisi_obj(f_obj, &x_j, obj_size);

                    /* Isprazni spremnik u datoteku nakon pisanja,
                       pri prijelazu s pisanja na citanje. */
                isprazni_spremnik(f_obj);
            }
        }

        if (l < j) {
                /* Zamjena x[l] i x[j], na pozicijama pos_l, pos_j. */
            postavi_fpos(f_obj, pos_j, SEEK_SET);
            napisi_obj(f_obj, &x_l, obj_size);
            postavi_fpos(f_obj, pos_l, SEEK_SET);
            napisi_obj(f_obj, &x_j, obj_size);

                /* Isprazni spremnik u datoteku nakon pisanja,
                   pri prijelazu s pisanja na citanje. */
            isprazni_spremnik(f_obj);
        }

        f_quick_sort(f_obj, l, j - 1L, obj_size, obj_cmp);
        f_quick_sort(f_obj, j + 1L, d, obj_size, obj_cmp);
    }

        /* Postavi se na pocetak datoteke (za svaki slucaj). */
/*         Ovo nema smisla u rekurzivnoj funkciji!
    rewind(f_obj);
 */

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    f_quick_sort(f_obj, 0, n_obj - 1, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
